Package: netgwas
Type: Package
Title: Network-Based Genome Wide Association Studies
Version: 1.14.5
Authors@R: c(person(given = "Pariya",
           family = "Behrouzi",
           role = c("aut", "cre"),
           email = "pariya.behrouzi@gmail.com", comment = c(ORCID = "0000-0001-6762-5433")), person(given = "Ernst", family = "C. Wit", role = "ctb"))
Depends: R (>= 3.1.0)
Imports: Matrix(>= 1.4-1), igraph, qtl, parallel, methods, glasso,
        MASS, huge,tmvtnorm
Suggests: testthat
Description: A multi-core R package that contains a set of tools based on copula graphical
             models for accomplishing the three interrelated goals in genetics and genomics in an
			 unified way: (1) linkage map construction, (2) constructing linkage disequilibrium
			 networks, and (3) exploring high-dimensional genotype-phenotype network and genotype-
			 phenotype-environment interactions networks. 
			 The 'netgwas' package can deal with biparental inbreeding and outbreeding species with
			 any ploidy level, namely diploid (2 sets of chromosomes), triploid (3 sets of chromosomes),
			 tetraploid (4 sets of chromosomes) and so on. We target on high-dimensional data where 
			 number of variables p is considerably larger than number of sample sizes (p >> n). 
			 The computations is memory-optimized using the sparse matrix output. The 'netgwas' 
			 implements the methodological developments in Behrouzi and Wit (2017)
             <doi:10.1111/rssc.12287> and Behrouzi and Wit (2017) <doi:10.1093/bioinformatics/bty777>. 
License: GPL-3
Encoding: UTF-8
Date/Publication: 2026-05-04 12:50:09 UTC
NeedsCompilation: no
Packaged: 2026-05-04 09:36:59 UTC; behro001
Author: Pariya Behrouzi [aut, cre] (ORCID:
    <https://orcid.org/0000-0001-6762-5433>),
  Ernst C. Wit [ctb]
Maintainer: Pariya Behrouzi <pariya.behrouzi@gmail.com>
Repository: CRAN
Built: R 4.6.0; ; 2026-05-04 17:19:45 UTC; unix

Package: netgwas
Type: Package
Title: Network-Based Genome Wide Association Studies
Version: 1.14.5
Authors@R: c(person(given = "Pariya",
           family = "Behrouzi",
           role = c("aut", "cre"),
           email = "pariya.behrouzi@gmail.com", comment = c(ORCID = "0000-0001-6762-5433")), person(given = "Ernst", family = "C. Wit", role = "ctb"))
Depends: R (>= 3.1.0)
Imports: Matrix(>= 1.4-1), igraph, qtl, parallel, methods, glasso,
        MASS, huge,tmvtnorm
Suggests: testthat
Description: A multi-core R package that contains a set of tools based on copula graphical
             models for accomplishing the three interrelated goals in genetics and genomics in an
			 unified way: (1) linkage map construction, (2) constructing linkage disequilibrium
			 networks, and (3) exploring high-dimensional genotype-phenotype network and genotype-
			 phenotype-environment interactions networks. 
			 The 'netgwas' package can deal with biparental inbreeding and outbreeding species with
			 any ploidy level, namely diploid (2 sets of chromosomes), triploid (3 sets of chromosomes),
			 tetraploid (4 sets of chromosomes) and so on. We target on high-dimensional data where 
			 number of variables p is considerably larger than number of sample sizes (p >> n). 
			 The computations is memory-optimized using the sparse matrix output. The 'netgwas' 
			 implements the methodological developments in Behrouzi and Wit (2017)
             <doi:10.1111/rssc.12287> and Behrouzi and Wit (2017) <doi:10.1093/bioinformatics/bty777>. 
License: GPL-3
Encoding: UTF-8
Date/Publication: 2026-05-04 12:50:09 UTC
NeedsCompilation: no
Packaged: 2026-05-04 09:36:59 UTC; behro001
Author: Pariya Behrouzi [aut, cre] (ORCID:
    <https://orcid.org/0000-0001-6762-5433>),
  Ernst C. Wit [ctb]
Maintainer: Pariya Behrouzi <pariya.behrouzi@gmail.com>
Repository: CRAN
Built: R 4.5.2; ; 2026-05-04 17:00:35 UTC; unix

export(simgeno)
export(simRIL)
export(cutoffs)
export(lower.upper)
export(R.gibbs)
export(R.approx)
export(netmap)
export(netsnp)
export(netphenogeno)
export(selectnet)
export(buildMap)
export(cal.pos)
export(detect.err)
export(cross2netgwas)
export(netgwas2cross)

S3method("plot","simgeno")
S3method("print","simgeno")
S3method("plot","netgwas")
S3method("print","netgwas")
S3method("plot","select")
S3method("plot","netgwasmap")
S3method("print","netgwasmap")

import("Matrix", "igraph", "qtl", "parallel")

importFrom( "tmvtnorm", "rtmvnorm.sparseMatrix", "ptmvnorm")
importFrom( "glasso", "glasso")
importFrom( "methods", "as", "new")
importFrom( "grDevices", "gray.colors", "terrain.colors", "rainbow")
importFrom( "graphics", "par", "plot", "legend", "screen", "split.screen", "plot.new", "box", "title")
importFrom( "stats", "cor", "dnorm", "pnorm", "qnorm", "runif", "rnorm", "rchisq", "sd", "rbinom" )
importFrom( "utils", "flush.console")
importFrom( "huge", "huge")
importFrom( "MASS", "sammon")

#-------------------------------------------------------------------------------#
# Package: Network-Based Genome-Wide Association Studies                        #
# Author: Pariya Behrouzi                                                       #
# Emails: <pariya.Behrouzi@gmail.com>                                           #
# Date: Nov 21th 2017                                                           #
#-------------------------------------------------------------------------------#

Gibbs_method = function(y, rho = NULL, n_rho = NULL, rho_ratio = NULL, Theta=NULL, ncores = 4, chain = 1, max.elongation = 10, em.tol=0.001) 
{
  p <- ncol(y)
  n <- nrow(y)
  lower.upper = lower.upper(y)

  if(is.null(rho))
  {
	if(is.null(n_rho)) n_rho = 10
	cr = cor(y, method="spearman") - diag(p)
	cr[is.na(cr)] <- 0
	rho_max = max(max(cr),-min(cr))
	if(rho_max == 0) 
	{
		ty <- npn(y, npn.func= "shrinkage")
		cr = cor(ty, method="spearman") - diag(p)
		rho_max = max(max(cr),-min(cr))
	}
	if(rho_max >= .7) rho_max = .7
	rho_min = rho_ratio * rho_max
	rho = exp(seq(log(rho_max), log(rho_min), length = n_rho))
	rm(cr, rho_max, rho_min, rho_ratio)
  }

  Gibbs.method <- calculate_EM_Gibbs(chain, y, rho = rho[chain], Theta=Theta, lower.upper = lower.upper, em.tol = em.tol, em.iter = max.elongation, gibbs.iter = 500, mc.iter = 1000, ncores = ncores)
  invisible(return(Gibbs.method))
}  

#-------------------------------------------------------------------------------#
# Package: Network-Based Genome-Wide Association Studies                        #
# Author: Pariya Behrouzi                                                       #
# Emails: <pariya.Behrouzi@gmail.com>                                           #
# Date: Nov 21th 2017                                                           #
#-------------------------------------------------------------------------------#

element_S_j = function(j, lower_upper, mu=0, sigma=1)
{
  	delta1 <- (lower_upper$lower[ ,j] - mu) / sigma
  	delta2 <- (lower_upper$upper[ ,j] - mu) / sigma
  	tmp1 <- (dnorm(delta1) - dnorm(delta2)) / (pnorm(delta2) - pnorm(delta1))
  	EX <- mu + tmp1 * sigma
  	
  	delta1[delta1 < -1e+10] <- -1e+10
  	delta2[delta2 > 1e+10] <- 1e+10
  	tmp2 <- (delta1*dnorm(delta1) - delta2*dnorm(delta2)) / (pnorm(delta2) - pnorm(delta1))
  	EXX <- sigma^2 + mu^2 + sigma^2 * tmp2 + 2 * mu * sigma * tmp1
	rm(delta1, delta2, tmp1, tmp2)
	gc()
	
	return(list(EX=EX, EXX=EXX))
}

approx_method = function(y, Z, ES=NULL, rho = NULL, lower_upper=NULL, chain=1, ncores=4, em_tol=.001, em_iter=10 )
{
	obj <- glasso(s=ES, rho=rho[chain], maxit=1000, penalize.diagonal=FALSE)
	Theta <- Matrix((t(obj$wi) + obj$wi) / 2, sparse = TRUE)
	Sigma <- (t(obj$w) + obj$w) / 2
	sd_marginal  <- sqrt(diag(Sigma))
	sd_marginal[abs(sd_marginal) < 1e-10] <- 1e-10
	Sigma <- diag(1/sd_marginal) %*% Sigma %*% diag(1/sd_marginal)
	Theta <- diag(sd_marginal) %*% Theta %*% diag(sd_marginal)
	rm(obj, sd_marginal)
	gc()

	approx.method <- calculate_EM_approx(chain=chain, y=y, Z=Z, rho= rho[chain], Sigma=Sigma, Theta= Theta, lower_upper=lower_upper, em_tol=em_tol, em_iter = em_iter, ncores=ncores)

	invisible(return(approx.method))
}

#-------------------------------------------------------------------------------#
# Package: Network-Based Genome-Wide Association Studies                        #
# Author: Pariya Behrouzi                                                       #
# Emails: <pariya.Behrouzi@gmail.com>                                           #
# Date: Nov 21th 2017                                                           #
#-------------------------------------------------------------------------------#

calculate_EM_Gibbs = function(chain, y, rho=rho[1], Theta=NULL, lower.upper, em.tol=.001, em.iter=10, c.em.iter=1, gibbs.iter=1000, mc.iter=1000, ncores = 4)
{
	n = nrow(y)
	p = ncol(y) 
	c.em.iter = 1
	dif  =  100
	while((c.em.iter <= em.iter) && (dif >= em.tol)) 
	{   
		R   <- calculate.R.internal( y, theta = Theta, lower.upper = lower.upper, gibbs.iter = gibbs.iter, mc.iter = mc.iter, ncores = ncores)
		R.gl <- glasso(s= R, rho, maxit=1000, penalize.diagonal=FALSE)
		if(det(R.gl$w) <= 0)
		{
		  R.gl$w <- nearPD(R.gl$w,keepDiag=TRUE)$mat 
		}
		dif <- sum(abs(Theta - R.gl$wi)/p^2) 
		Theta = as(R.gl$wi, "dgTMatrix") 
		Theta = as(Theta, "sparseMatrix") 
		c.em.iter <- c.em.iter + 1
	}
	  
	results <- list()
	results$Theta	<- Matrix(Theta, sparse=TRUE)
	results$Sigma	<- Matrix(R.gl$w, sparse=TRUE)
	results$ES		<- Matrix(R, sparse = TRUE)
	results$Z		<- NULL
	results$rho		<- rho
	results$loglik	<- n/2 *(determinant(results$Theta, logarithm = TRUE)$modulus - sum(diag(results$ES %*%results$Theta)))

	return(results)
}

##################################################################
####### Convert netgwas.map object to cross object in qtl
#################################################################
as.cross <- function(mp, pop.type= "DH") 
{    
  RILN <- paste("RIL",1:20, sep = "")
  allow.pop <- c("BC","DH","ARIL",RILN)
  if(!(pop.type %in% allow.pop))
    stop("Population type needs to be \"BC\",\"DH\",\"ARIL\" or \"RILn\" (see ?cal.pos).")
  ptype <- pop.type
  
  obj = list( )
  obj$geno = vector("list", length = length(table(mp$map[, 2]))) #max(mp$map[, 2]))
  obj$pheno <- vector("list", length = 1)
  
  dat <- mp$allres$data
  if(is.null(dimnames(dat)[[1]])) dimnames(dat)[[1]] <- paste("ind", 1:dim(dat)[1], sep="")
  obj$pheno <- data.frame(Genotype = factor(dimnames(dat)[[1]]))
  rownames(dat) <- NULL
  
  for(i in 1: length(obj$geno))
  {
    
    obj$geno[[i]]$data <- vector("list", length = 1)
    obj$geno[[i]]$map <-  vector("list", length = 1)
    
    mar.chr <- cumsum(table(mp$map[, 2]))
    
    if(i == 1) start <- 1 else start <- mar.chr[i - 1] + 1
    obj$geno[[i]]$data <- dat[  , start:mar.chr[i]]
    
    mar.order <- as.character(mp$map$markers[start:mar.chr[i]])
    obj$geno[[i]]$map <- (1:length(as.character(mp$map$markers[start:mar.chr[i]]))) + 0.2 ##fake positions
    names(obj$geno[[i]]$map) <- mar.order
    
    class(obj$geno[[i]]) <- "A"
  }
  names(obj$geno) <- 1:length(mar.chr)
  wp <- (1:23)[allow.pop %in% ptype]
  class(obj) <- c(c("bc","dh","riself",rep("f2",20))[wp],"cross")
  if(wp %in% 4:23) co <- convert2bcsft(obj, F.gen = wp - 3, estimate.map = FALSE)
  
  return(obj)
}

quickEst <- function(object, chr, map.function = "kosambi", ...){
  if (!any(class(object) == "cross"))
    stop("Input should have class \"cross\".")
  if (missing(chr))
    chr <- names(nmar(object))
  imf <- switch(map.function, kosambi = imf.k, haldane = imf.h,
                morgan = imf.m, cf = imf.cf)
  nm <- nmar(object)
  for(i in chr){
    temp <- subset(object, chr = i)
    if(nmar(temp) != 1){
      est <- est.rf(temp)$rf
      nc <- dim(est)[1]
      er <- est[cbind(2:nc,1:(nc - 1))]
      temp$geno[[i]]$map <- c(0,cumsum(imf(er)))
      names(temp$geno[[i]]$map) <- dimnames(temp$geno[[i]]$data)[[2]]
      tempa <- argmax.geno(temp, step = 0, map.function = map.function, ...)
      tempa$geno[[i]]$data <- tempa$geno[[i]]$argmax
      tempa$geno[[i]] <- tempa$geno[[i]][-3]
      esta <- est.rf(tempa)$rf
      era <- esta[cbind(2:nc,1:(nc - 1))]
      if(class(object)[1] == "riself")
        era <- (era/2)/(1 - era)
      object$geno[[i]]$map <- c(0,cumsum(imf(era)))
      names(object$geno[[i]]$map) <- dimnames(object$geno[[i]]$data)[[2]]
    }
  }
  object
}

cal.pos <- function(netgwasmap, pop.type= NULL, map.func = "haldane", chr ){
  if(is.null(pop.type)) stop("Population type needs to be \"BC\",\"DH\",\"ARIL\" or \"RILn\" ")
  
  map <- as.cross(netgwasmap, pop.type= pop.type)  
  map <- quickEst( map, chr, map.function = map.func)
  map <- jittermap(map, amount=1e-6)
  return(map)
}

#-------------------------------------------------------------------------------#
# Package: Network-Based Genome-Wide Association Studies                        #
# Authors: Pariya Behrouzi, Danny Arends                                        #
# maintainer: <pariya.Behrouzi@gmail.com>                                       #
# Date: Nov 21th 2017                                                           #
#-------------------------------------------------------------------------------#

calcRcore <- function(i, n, p, gibbs.iter, mc.iter, theta, lower.upper, verbose = TRUE){
	z.gibbs <- rtmvnorm.sparseMatrix(n=mc.iter, H= theta , lower=lower.upper$lower[i,], upper=lower.upper$upper[i,], burn.in=gibbs.iter)
    summed <- matrix(0, p, p)
    for(iteration in 1: mc.iter)
    { 
      summed <- summed + (z.gibbs[iteration,] %*% t(z.gibbs[iteration,]))
    }
    R <- summed /  mc.iter
    return(R)
}
   
calculate.R.internal = function(y, theta=NULL, lower.upper=NULL, gibbs.iter=1000, mc.iter=1000, ncores = 4, verbose = TRUE)
{
  if(missing(y)) stop("argument \"y\" is missing, with no default")
  S <- 0
  p <- ncol(y)
  n <- nrow(y)
  if(missing(lower.upper)) lower.upper <- lower.upper(y)
  if(ncores > 1){
    cl <- makeCluster(ncores)
    scovs <- parLapply(cl = cl, 1:n, function(i) { 
        calcRcore(i, n, p, gibbs.iter, mc.iter, theta, lower.upper, verbose); 
    })
    stopCluster(cl)
  }else{
    scovs <- lapply(1:n, function(i){ calcRcore(i, n, p, gibbs.iter, mc.iter, theta, lower.upper); })
  }
  
  for(i in 1:n){ 
    S <- S + scovs[[i]]
  }
  ES  <- cov2cor(S/n)
  rm(S)
  return(ES)
}

R.gibbs = function(y, theta, gibbs.iter=1000, mc.iter=500, ncores=NULL, verbose = TRUE)
{
  if(is.null(ncores)) ncores= detectCores() - 1
  if(missing(theta)) 
  {
	theta <- sparseMatrix(i = 1:ncol(y), j = 1:ncol(y), x = 1)
  }else{
	theta <- as(theta, "dgTMatrix") 
	theta <- as(theta, "sparseMatrix") 
  }
  lower.upper <- lower.upper(y)
  ES <- calculate.R.internal(y, theta, lower.upper, gibbs.iter, mc.iter, ncores, verbose)
  return(ES = ES)
 }

netgwas2cross <- function(netgwasmap, pop.type= NULL, map.func = "haldane" ){
  if(is.null(pop.type)) stop("Population type needs to be \"BC\",\"DH\",\"ARIL\" or \"RILn\" ")
  map <- as.cross(netgwasmap, pop.type= pop.type)  
  map <- quickEst( map, map.function = map.func)
  map <- jittermap(map, amount=1e-6)
  return(map)
}

#-------------------------------------------------------------------------------#
# Package: Network-Based Genome-Wide Association Studies                        #
# netmap(): Reconstruct conditional dependence networks among markers on genome #
# Authors: Pariya Behrouzi, Ernst Wit                                           #
# maintainer: <pariya.Behrouzi@gmail.com>                                       #
# Date: Nov 21th 2017                                                           #
# Version: 0.0.1-1                                                              #
#-------------------------------------------------------------------------------#

netmap = function(data, method = "npn", cross= NULL, rho = NULL, n.rho = NULL, rho.ratio = NULL, min.m= NULL, use.comu= FALSE, ncores = "all", em.iter = 5, verbose = TRUE) 
{
  gcinfo(FALSE)
	if(is.null(cross)) stop("the argument cross should be fill in either as \"inbred\" or \"outbred\" \n")
	if(!is.matrix(data)) data <- as.matrix(data)
	if(ncores == "all") ncores <- detectCores() - 1
	if(is.null(em.iter)) em.iter = 5
	em.tol = 0.001
	
	result = list()
	data <- cleaning.dat(data)
	p = ncol(data)
	n = nrow(data)
	
if( method == "gibbs" ||  method== "approx" ) 
{
	if( method == "gibbs")
	{
		if((is.null(rho)) && (is.null(n.rho)) ) n.rho = 6
		if(! is.null(rho)) n.rho  = length(rho)
		if(is.null(rho.ratio)) 
		{
			if(p >= 2000) rho.ratio = 0.5 else rho.ratio = 0.45
		}
		est = vector("list", n.rho)
		for(chain in 1 : n.rho ) 
		{
			if(verbose)
			{
				m <- paste(c("Constructing linkage map is in progress:", floor(100 * chain/n.rho), "%"), collapse="")
				cat(m, "\r")
				flush.console()
			}
			if( chain == 1)
			{
				est[[chain]] = vector("list", n.rho)
				Theta = sparseMatrix(i = 1:ncol(data), j = 1:ncol(data), x = 1)
				est[[chain]] = Gibbs_method(data, rho=rho, n_rho=n.rho, rho_ratio=rho.ratio, Theta = Theta, ncores = ncores, chain = chain, max.elongation = em.iter, em.tol=em.tol)
			}else{
				est[[chain]] = vector("list", n.rho)
				Theta = est[[(chain - 1)]]$Theta
				Theta = as(Theta, "dgTMatrix") 
				Theta = as(Theta, "sparseMatrix")
				est[[chain]] = Gibbs_method(data, rho=rho, n_rho=n.rho, rho_ratio=rho.ratio, Theta= Theta, ncores = ncores, chain = chain, max.elongation = em.iter, em.tol=em.tol)
			}
		}
		rm(Theta)
		gc()
	}
	
	if(method == "approx")
	{

		if( !is.null(rho) ) n.rho = length(rho) 
		if( is.null(n.rho) ) n.rho = 6
		if(is.null(rho.ratio)) 
		{
			if(p >= 2000) rho.ratio = 0.7 else rho.ratio = 0.65
		}
		
		ini = initialize(data, rho = rho, n_rho = n.rho, rho_ratio = rho.ratio, ncores=ncores )
		rho = ini$rho
		Z	= ini$Z
		ES	= ini$ES
		lower_upper = ini$lower_upper
		
		rm(ini)
		gc()
		
		est <- vector("list", n.rho)
		for(chain in 1 : n.rho) 
		{
		if(verbose)
			{
				m <- paste(c("Constructing linkage map is in progress:", floor(100 * chain/n.rho), "%"), collapse="")
				cat(m, "\r")
				flush.console()
			}
			est[[chain]] <- vector("list", n.rho)
			est[[(chain)]] <- approx_method(data, Z, ES=ES, rho=rho, lower_upper=lower_upper, chain = chain, ncores = ncores, em_tol=em.tol, em_iter=em.iter)
		}

		rm(lower_upper)
		gc()
	}

	result$Theta  = vector("list", n.rho)
	result$path   = vector("list", n.rho)
	result$ES	  = vector("list", n.rho)
	result$Z	  = vector("list", n.rho)
	result$rho	  = vector()
	result$loglik = vector()
	result$data	  = data
	rm(data)
	
	for(chain in 1:n.rho)
	{
		result$Theta[[chain]]	= est[[chain]]$Theta
		if(!is.null(colnames(result$data))) colnames(result$Theta[[chain]]) = colnames(result$data)
		result$path[[chain]]	= abs(sign(result$Theta[[chain]])) - Diagonal(p)
		result$rho[chain]		= est[[chain]]$rho
		result$loglik[chain]	= est[[chain]]$loglik
	}
	rm(est)
	class(result) = "netgwas"
}else{	
	if(method == "npn")
	{
		if(verbose)
		{
			m <- paste("Constructing linkage map is in progress ... \n")
			cat(m, "\r")
			flush.console()
		}
		if((is.null(rho)) && (is.null(n.rho)) ) n.rho = 6
		if(! is.null(rho)) n.rho  = length(rho)
		if(is.null(rho.ratio)) 
		{
			rho.ratio = 0.5
		}
		if( any(is.na(data)) ) npn.func = "shrinkage" else npn.func = "skeptic"
		tdata <- npn(data, npn.func= npn.func)
		if(is.null(rho))
		{
			cr = cor(tdata, method = "spearman") - diag(p)
			cr[is.na(cr)] <- 0
			rho_max = max(max(cr),-min(cr))
			if(( p <= 100) &&(rho_max >= .7)) rho_max = .8 
			if(( p > 100) && (rho_max >= .7)) rho_max = .7 
			rho_min = rho.ratio * rho_max
			rho = exp(seq(log(rho_max), log(rho_min), length = n.rho))
			rm(cr, rho_max, rho_min, rho.ratio)
		}
		
		est <- huge(tdata, lambda= rho, nlambda=n.rho, lambda.min.ratio=rho.ratio, method="glasso", verbose=FALSE)
			
		result$Theta  = est$icov
		result$path   = est$path
		result$rho	  = est$lambda
		result$loglik = n/2 * (est$loglik)
		result$data	  = data
		
		rm(data, est)		
		if(is.null(colnames(result$Theta[[1]]))) {for(i in 1:length(result$rho)) colnames(result$Theta[[i]]) <- colnames(result$data)}
		if(is.null(colnames(result$path[[1]]))) {for(i in 1:length(result$rho)) colnames(result$path[[i]]) <- colnames(result$data)}
		class(result) = "netgwas"
	}	
}	
	#Detecting LGs and ordering within each LG
	if(is.null(use.comu)) use.comu <- FALSE
	sel.net <- selectnet(result, criteria = "ebic", ebic.gamma = 0.5, ncores = ncores, verbose=FALSE)
	if(method == "npn") colnames(sel.net$opt.theta) <- colnames(result$data)
	map <- buildMap.internal(network= sel.net$opt.theta, cross= cross, num.iso.m= min.m, use.comu= use.comu)

	result$data <- result$data[ , c(as.character(map[,1]))]
	if(cross == "outbred") map <- data.frame(map) #added due to cross obj as.cross fun (can be removed later)
	results <- list( map= map, opt.index= sel.net$opt.index, cross.typ= cross, allres= result, man= FALSE)
	
	class(results$allres) = "netgwas"
	class(results) = "netgwasmap"
	rm(result)
	
	cat("Constructing linkage map is done.             \r\n")
	return(results)
}

#-----------------------------------------------------#
#   		Plot for class "netgwasmap"      	      #
#-----------------------------------------------------#
plot.netgwasmap = function(x, vis= NULL, layout= NULL, vertex.size= NULL, label.vertex = "none", label.size= NULL, vertex.color= NULL, edge.color = "gray29", sel.ID = NULL, ...)
{
    if(! inherits( x, "netgwasmap" )) stop("netgwas.object should belong to the netgwas class. \n ")
	if(is.null(vis)) vis <- "summary"
	if(is.null(label.vertex)) label.vertex <- "none"
	if(vis == "summary")
	{
		if(is.null( vertex.color))  vertex.color <- "red"
		if(is.null( label.size)) label.size <- 0.8
		if(is.null( vertex.size)) vertex.size <- 5
		if(label.vertex == "none") vertex.label <- NA

		opt.theta <- as.matrix(x$allres$Theta[[x$opt.index]]) 
		p <- ncol(opt.theta)
		path <-  as.matrix(abs(sign(opt.theta)) - diag(rep(1,p)))
		adj <- graph.adjacency(path, mode="undirected")
		adj$label.cex <- label.size
		if(label.vertex == "all") vertex.label <- colnames(opt.theta) #else{vertex.label <- NA}
		if(is.null(layout)) layout <- layout.fruchterman.reingold
    
		par(mfrow=c(1,1))
		split.screen( figs = c( 1, 2 ) )
		split.screen( figs = c( 1, 1 ), screen = 1 )
		split.screen( figs = c( 2, 1 ), screen = 2 )
		screen(1)
		plot(adj, layout=layout, edge.curved = F, vertex.label= vertex.label, vertex.color=vertex.color, edge.color="gray40", vertex.size=vertex.size, vertex.label.dist=0, vertex.label.color="darkblue", main="Network")
		screen(4)
		#image(path, col = gray.colors(256), xaxt="n", yaxt="n", main= "Conditional dependence relationships \nbefore ordering", cex.main=.8, cex.lab=.8, cex.axis=.8)
		image(path, col = c("white", "black"), xaxt="n", yaxt="n", main= "Before ordering markers", cex.main=1.5, cex.lab=1.5, cex.axis=1.5)
		title(ylab = "markers", cex.lab = 1.5, line = .5)
		title(xlab = "markers", cex.lab = 1.5, line = .5)

		index <- as.character(x$map[ ,1])
		rownames(path) <- colnames(path)
		path.After <- path[c(index), c(index)] 
		screen(5)
		#image(path.After, col = gray.colors(256), xaxt="n", yaxt="n", ,main="Conditional dependence relationships \nafter ordering", cex.main=0.8, cex.lab=.8, cex.axis=.8)
		image(path.After, col = c("white", "black"), xaxt="n", yaxt="n", ,main="After ordering markers", cex.main=1.5, cex.lab=1.5, cex.axis=1.5)
		title(ylab = "markers", cex.lab = 1.5, line = .5)
		title(xlab = "markers", cex.lab = 1.5, line = .5)
	}
	if(vis == "interactive")
	{
		 adj <- as.matrix(x$allres$path[[x$opt.index]])
		  
		  if(is.null(vertex.size)) vertex.size <- 5
		  if(is.null(vertex.color)) vertex.color <- "red"
		  if(is.null(edge.color)) edge.color <- "gray29"
		  if(is.null(sel.ID)) sel.ID <- NULL
		  if((label.vertex == "some") && (is.null(sel.ID )) ) stop("Please select some vertex label(s) or fix label.vertex to either none or all.")

		  p <- ncol(adj)
		  A <- graph.adjacency(adj, mode= "undirected")
		  if(is.null(layout)) layout <- layout_with_fr(A)
		  #if(is.null(layout)) layout <- layout_with_kk(A, maxiter = 50 * p )
		  V(A)$label.cex <- 1
	 
		  if(label.vertex == "none")
		{
			V(A)$label <- NA
			tkplot(A, layout=layout, vertex.color=vertex.color, edge.color=edge.color, vertex.size=vertex.size, vertex.label.dist=0)  
		}

		  if(label.vertex == "some") 
		{
			V(A)$label <- colnames(adj)
			tkplot(A, vertex.label=ifelse(V(A)$label %in% sel.ID, V(A)$label, NA ), layout=layout, vertex.color=vertex.color, edge.color=edge.color, vertex.size=vertex.size, vertex.label.dist=0)
		}
		 if(label.vertex == "all") 
		{	
			V(A)$label <- colnames(adj)
			tkplot(A, vertex.label=colnames(adj) , layout=layout, vertex.color=vertex.color, edge.color=edge.color, vertex.size=vertex.size, vertex.label.dist=0)  
		}	
	}
	
	if(vis == "unordered markers") 
	{
	  opt.theta <- x$allres$Theta[[x$opt.index]]
	  print(image(Matrix(opt.theta), xlab="markers", ylab="markers", main="Unordered markers", cex= 3, sub= ""))
	}

	if(vis == "ordered markers") 
	{
	  opt.theta <- x$allres$Theta[[x$opt.index]]
	  rownames(opt.theta)  <- colnames(opt.theta) 
	  
	  #orderedm <- Matrix( opt.theta[colnames(x$allres$data)  , colnames(x$allres$data) ] )
	  map <- as.character(x$map[ ,1])
	  orderedm <- Matrix( opt.theta[ map  , map  ] )
	 
	  print(image(orderedm,  xlab="markers", ylab="markers", main="Ordered markers", cex= 3, sub=""))
	}
}

#-----------------------------------------------------#
#   		Summary for class "netgwasmap"                #
#-----------------------------------------------------#
print.netgwasmap = function(x, ...){
	cat("Number of linkage groups: ", length(unique(sort(x$map[,2]))), "\n")
	cat("Number of markers per linkage group: ", table(x$map[, 2]), "\n")
	cat("Total number of markers in the linkage map:", length(x$map[,1]),".", " (", (ncol(x$allres$Theta[[1]]) - length(x$map[,1])), " markers removed from the input genotype data) \n")
	cat("Number of sample size: n =", nrow(x$allres$data), "\n")
	cat("Number of categories in dataset:", length(unique(sort(as.matrix(x$allres$data))))," (" , unique(sort(as.matrix(x$allres$data))), ")", "\n")
	cat("The estimated linkage map is inserted in <OUTPUT NAME>$map \n")
	cat("To visualize the network consider plot(<OUTPUT NAME>) \n")
	cat("----------------------- \n")
	cat("To visualize the other associated networks consider plot(<OUTPUT NAME>$allres) \n")
	if(x$man == FALSE) cat("To build a linkage map for your desired network consider buildMap() function \n")
}

context("netmap")

test_that("npn map construction method works", 
          {
            data(CviCol)
            M1 <- netmap(CviCol, method="npn", cross= "inbred", rho = 0.4)
            #dput(M1$map$LG)
            
            expect_equal( max(M1$map$LG), 5 )  
          })


test_that("Gibbs map construction method works ", 
  {
	 data(CviCol)
	 D2 <- CviCol[ 1:20, c(1:5, 61:65)]
	 M2 <- netmap(D2, method="gibbs", cross= "inbred", rho = 0.3, ncores = 1)
	
	 expect_equal( max(M2$map$LG), 2)  
   })


test_that("Approx map construction method works ", 
  {
	 data(CviCol)
	 D3 <- CviCol[ 1:5, 1:5 ]
	 M3 <- netmap(D3, method="approx", cross= "inbred", rho = 0.6, ncores = 1)
	
	 expect_equal( max(M3$map$LG), 2 )  
   })

context("buildMap")

test_that("buildMap construction method works ", 
  {
    data(CviCol)
    M1 <- netmap(CviCol, method="npn", cross= "inbred")
    bm <- buildMap(M1, opt.index=5)	 
    #dput(bm$map$markers)
	
    expect_equal( max(bm$map$LG), 5 )
   })

###################################################################
### Convert cross object from R\qtl package to netgwas object #####
###################################################################

cross2netgwas <- function(cross.obj)
{
  nchr <- length(cross.obj$geno)
  netgwas.obj <- cross.obj$geno[[1]]$data
  
  if(nchr == 1) return(netgwas.obj) else{ 
      for(i in 2: nchr ) {
      netgwas.obj <- cbind(netgwas.obj, cross.obj$geno[[i]]$data)
      }
  return(netgwas.obj)
  }
  
  #pos.map$geno[[i]]$map
}

#-------------------------------------------------------------------------------#
# Package: Network-Based Genome-Wide Association Studies                        #
# Author: Pariya Behrouzi                                                       #
# Emails: <pariya.Behrouzi@gmail.com>                                           #
# Date: Nov 21th 2017                                                           #
#-------------------------------------------------------------------------------#

lower.upper = function(y){
	cutoffs <- cutoffs(y)
	levels	<- unique(sort(unlist(y)))
	n.levels<- length(levels)
	n <- nrow(y)
	p <- ncol(y)
	lower = matrix(nrow=n,ncol=p)
	upper = matrix(nrow=n,ncol=p)
	for (i in 1:n){
		sel <- match(y[i,],levels)
		lower[i,]<-apply(cbind(sel,cutoffs),1,function(x){x[x[1]+1]})
		upper[i,]<-apply(cbind(sel,cutoffs),1,function(x){x[x[1]+2]})
	}
	lower[is.na(lower)] <- -Inf 
	upper[is.na(upper)] <- Inf	
	
	return(list(lower=lower,upper=upper))
}

#-----------------------------------------------------------------------------------------#
# Package: Network-Based Genome-Wide Association Studies                        		  #
# netsnp(): Reconstruct intra- and inter chromosomal interaction (linkage disequilibrium) #
# Authors: Pariya Behrouzi, Ernst Wit                                                     #
# maintainer: <pariya.Behrouzi@gmail.com>                                                 #
# Date: Nov 21th 2017                                                                     #
# Version: 0.0.1-1                                                                        #
#-----------------------------------------------------------------------------------------#

netsnp = function(data, method = "gibbs", rho = NULL, n.rho = NULL, rho.ratio = NULL, ncores = 1, em.iter = 5, em.tol = .001, verbose = TRUE) 
{
  gcinfo(FALSE)
	if(!is.matrix(data)) data <- as.matrix(data)
	if(ncores == "all") ncores <- detectCores() - 1
	if(is.null(em.iter)) em.iter = 5
	if(is.null(em.tol)) em.tol = 0.001
	
	result = list()
	data <-cleaning.dat(data)
	n = nrow(data)
	p = ncol(data)
			
if( method == "gibbs" ||  method== "approx" )
{
	if( method == "gibbs")
	{
		if((is.null(rho)) && (is.null(n.rho)) ) n.rho  = 6
		if(! is.null(rho)) n.rho  = length(rho)
		if(is.null(rho.ratio)) rho.ratio = 0.15 
			
		est = vector("list", n.rho)
		for(chain in 1 : n.rho ) 
		{
		if(verbose)
		{
			m <- paste(c("Reconstructing intra- and inter- chromosomal interactions network is in progress:", floor(100 * chain/n.rho), "%"), collapse="")
			cat(m, "\r")
			flush.console()
		}
			if( chain == 1)
			{
				est[[chain]] = vector("list", n.rho)
				Theta = sparseMatrix(i = 1:ncol(data), j = 1:ncol(data), x = 1)
				est[[chain]] = Gibbs_method(data, rho=rho, n_rho=n.rho, rho_ratio=rho.ratio, Theta = Theta, ncores = ncores, chain = chain, max.elongation = em.iter, em.tol=em.tol)
			}else{
				est[[chain]] = vector("list", n.rho)
				Theta = est[[(chain - 1)]]$Theta
				Theta = as(Theta, "dgTMatrix") 
				Theta = as(Theta, "sparseMatrix")
				est[[chain]] = Gibbs_method(data, rho=rho, n_rho=n.rho, rho_ratio=rho.ratio, Theta= Theta, ncores = ncores, chain = chain, max.elongation = em.iter, em.tol=em.tol)
			}
		}
		rm(Theta)
		gc()
	}
	
	if(method == "approx")
	{
		if( !is.null(rho) ) n.rho = length(rho) 
		if( is.null(n.rho) ) n.rho = 6
		if(is.null(rho.ratio)) rho.ratio = 0.22
		
		ini = initialize(data, rho = rho, n_rho = n.rho, rho_ratio = rho.ratio, ncores=ncores )
		rho = ini$rho
		Z	= ini$Z
		ES	= ini$ES
		lower_upper = ini$lower_upper
		
		rm(ini)
		gc()
		
		est <- vector("list", n.rho)
		for(chain in 1 : n.rho) 
		{
		if(verbose)
			{
				m <- paste(c("Reconstructing intra- and inter- chromosomal interactions network is in progress:", floor(100 * chain/n.rho), "%"), collapse="")
				cat(m, "\r")
				flush.console()
			}
			est[[chain]] <- vector("list", n.rho)
			est[[(chain)]] <- approx_method(data, Z, ES=ES, rho=rho, lower_upper=lower_upper, chain = chain, ncores = ncores, em_tol=em.tol, em_iter=em.iter)
		}

		rm(lower_upper)
		gc()
	}

	result$Theta  = vector("list", n.rho)
	result$path   = vector("list", n.rho)
	result$ES	  = vector("list", n.rho)
	result$Z	  = vector("list", n.rho)
	result$rho	  = vector()
	result$loglik = vector()
	result$data	  = data
	rm(data)
	
	for(chain in 1:n.rho)
	{
		result$Theta[[chain]]	= est[[chain]]$Theta
		if(!is.null(colnames(result$data))) colnames(result$Theta[[chain]]) = colnames(result$data)
		result$path[[chain]]	= abs(sign(result$Theta[[chain]])) - Diagonal(p)
		result$ES[[chain]]		= est[[chain]]$ES
		result$Z[[chain]]		= est[[chain]]$Z
		result$rho[chain]		= est[[chain]]$rho
		result$loglik[chain]	= est[[chain]]$loglik
	}
	rm(est)
	
	cat("Reconstructing intra- and inter- chromosomal interactions network done.    \n")
}else{
	if( method == "npn" ){
	
		if(verbose)
		{
			m <- paste("Reconstructing intra- and inter- chromosomal interactions network is in progress ... \n")
			cat(m, "\r")
			flush.console()
		}
			
		if((is.null(rho)) && (is.null(n.rho)) ) n.rho  = 6
		if(! is.null(rho)) n.rho  = length(rho)
		if(is.null(rho.ratio)) rho.ratio = 0.14

		
		if( any(is.na(data)) ) npn.func = "shrinkage" else npn.func = "skeptic"
		tdata <- npn(data, npn.func= npn.func)
		est <- huge(tdata, lambda= rho, nlambda=n.rho, lambda.min.ratio=rho.ratio, method="glasso", verbose=FALSE)
			
		result$Theta  = est$icov
		result$path   = est$path
		result$rho	  = est$lambda
		result$loglik = n/2 * (est$loglik)
		result$data  = data
		rm(est)
		if(verbose)
		{
			m <- paste("Reconstructing intra- and inter- chromosomal interactions network done. \n")
			cat(m, "\r")
			flush.console()
		}
	}
}	
	class(result) = "netgwas"
	
	return(result)
}

#-----------------------------------------------------#
#   		Plot for class "netgwas"      	          #
#-----------------------------------------------------#
plot.netgwas = function( x, n.markers=NULL , ...)
{
	if(length(x$rho) == 1 ) par(mfrow = c(1, 2), pty = "s", omi=c(0.3,0.3,0.3,0.3), mai = c(0.3,0.3,0.3,0.3))
	if(length(x$rho) == 2 ) par(mfrow = c(2, 2), pty = "s", omi=c(0.3,0.3,0.3,0.3), mai = c(0.3,0.3,0.3,0.3))
	if(length(x$rho) >= 3 ) par(mfrow = c(2, ceiling(length(x$rho)/2)), pty = "s", omi=c(0.3,0.3,0.3,0.3), mai = c(0.3,0.3,0.3,0.3))
	
	for(chain in 1:length(x$rho))
	{
			if(length(x$rho) == 1 ) image(as.matrix(x$path[[chain]]), col = gray.colors(256), xlab= "", ylab="" ,main=paste( "rho ", x$rho[chain],  sep=""))
			if(length(x$rho) == 2 ) image(as.matrix(x$path[[chain]]), col = gray.colors(256), xlab= "", ylab="" ,main=paste( "rho ", x$rho[chain],  sep=""))
			
			adj = graph.adjacency(as.matrix(x$path[[chain]]), mode="undirected", diag=FALSE)
			if(is.null(n.markers)) 
			{
					memberships = 1
					vertex.color = "red" 
			}else{
				LG = length(n.markers)
				memberships = NULL
				i = 1
				while( i <= LG){
					grp <- rep(i, n.markers[i])
					memberships = c(memberships, grp)
					i = i + 1
				}
				if(chain == 1){
					color = sample(terrain.colors(max(memberships) + 10), max(memberships))
					cl = color[memberships]
				}
				vertex.color = cl
			}
			adj$layout	= layout.fruchterman.reingold
			plot(adj, vertex.color = vertex.color, edge.color='gray40', vertex.size = 7, vertex.label = NA , vertex.label.dist = NULL)
	}
	if(length(memberships) > 1) legend("bottomright", paste("group", 1:length(n.markers)), cex=0.7, col= color, pch=rep(20,10))		
}


#-----------------------------------------------------#
#   		Summary for class "netgwas"        		  #
#-----------------------------------------------------#
print.netgwas = function(x, ...){
	cat("Estimated a graph path for", length(x$rho), "penalty term(s)" , "\n")
	cat("Number of variables: p =", ncol(x$data), "\n")
	cat("Number of sample size: n =", nrow(x$data), "\n")
	sparsLevel <- round(sapply(1:length(x$rho), function(i) sum(x$Theta[[i]])/ncol(x$data)/(ncol(x$data)-1)),4)
    cat("Sparsity level for each graph in the path:", sparsLevel,"\n")
	cat("To visualize the graph path consider plot() function \n")
	cat("To SELECT an optimal graph consider selectnet() function \n")
}

context("netsnp")

test_that("npn in netsnp works fine ", 
  {
	 data(CviCol)
	 L.M1 <- netsnp(CviCol[ 1:20, c(1:5, 61:65)], method="npn", rho = 0.3)$loglik
	
	  expect_equal( L.M1, -57.50442  )  
   })


test_that("Gibbs in netsnp works fine ", 
  {
	  data(CviCol)
	  L.M2 <- netsnp(CviCol[ 1:5, c(1:5, 61:65)], method="gibbs", rho = 0.8, ncores = 1)$loglik
	
	  expect_equal( L.M2,  -25  )  
   })


test_that("Approx in netsnp works fine ", 
  {
	data(CviCol)
	L.M3 <- netsnp(CviCol[ 1:5, c(1:5, 61:65)], method="approx", rho = 0.8, ncores = 1)$loglik
	
	 expect_equal( L.M3, -25 )  
   })

context("selectnet")

test_that("Model selection works fine ", 
          {
            D <- simgeno(p=10, n=30, k= 3, adjacent = 3, alpha = 0.06 , beta = 0.06)$data
            out  <-  netsnp(D, n.rho= 4, method="npn", ncores= 1)
            sel.ebic <- selectnet(out, criteria = "ebic")$opt.index 
            
            expect_equal(  sel.ebic, 4 )  
          })

#-------------------------------------------------------------------------------#
# Package: Network-Based Genome-Wide Association Studies                        #
# Author: Pariya Behrouzi                                                       #
# Emails: <pariya.Behrouzi@gmail.com>                                           #
# Date: Nov 21th 2017                                                           #
#-------------------------------------------------------------------------------#

cutoffs = function(y){
	p<-ncol(y)
	n<-nrow(y)
	k<-unique(sort(unlist(y)))
	n.levels<-length(k)
	q<-matrix(nrow=p,ncol=n.levels)
	for(i in 1:p){
		X=factor(y[,i],levels=k)
		No<-tabulate(X, nbins=n.levels)
		q[i,]<-qnorm(cumsum(No)/n)
	}
	q[ ,n.levels] <- Inf
	q<-cbind(-Inf,q)
	return(q)
}

#-------------------------------------------------------------------------------#
# Package: Network-Based Genome-Wide Association Studies                        #
# Author: Pariya Behrouzi                                                       #
# Emails: <pariya.Behrouzi@gmail.com>                                           #
# Date: Nov 21th 2017                                                           #
#-------------------------------------------------------------------------------#
model.selection = function( result, criterion, lower.upper, ebic.gamma = 0.5,  ncores = 1, loglik_Y=TRUE, verbose = TRUE)
{
	#if(class(result) == "netgwas") 
	if( inherits(result, "netgwas") )  
	{
		p	 = ncol(result$path[[1]])
		nrho = length(result$rho)
		n	 = nrow(result$data)

	}else{
		stop("netgwas.object should belong to the netgwas class. \n ")
	} 
	loglik = result$loglik
	
	result$df <- sapply(1:nrho, function(x) sum(as.matrix(result$Theta[[x]])[upper.tri(result$Theta[[x]])] != 0 ))
	if (criterion == "ebic")
	{
		if(verbose)
		{
			cat("Calculating extended Bayesian information criterion (ebic) selection ...")
			flush.console()
		}
		result$ebic.scores	= - 2 * loglik + ( log(n) * result$df ) + ( 4 * ebic.gamma * log(p) * result$df )
		result$opt.index	= which.min(result$ebic.scores)
		result$opt.theta	= result$Theta[[result$opt.index]]
		colnames(result$opt.theta) = colnames(result$data)
		result$opt.adj		= abs(sign(as.matrix(result$opt.theta))) - diag(rep(1,p))
		colnames(result$opt.adj) = colnames(result$opt.theta)
		result$opt.rho		= result$rho[result$opt.index]
	}
	
	if (criterion == "aic")
	{
		if(verbose)
		{
			cat("Calculating Akaike information criterion (AIC) selection ...")
			flush.console()
		}
		 
		result$aic.scores	= ( - 2 * loglik ) + ( 2 * result$df )
		result$opt.index	= which.min(result$aic.scores)
		result$opt.theta	= result$Theta[[result$opt.index]]
		colnames(result$opt.theta) = colnames(result$data)
		result$opt.adj		= abs(sign(result$opt.theta)) - diag(rep(1,p))
		colnames(result$opt.adj) = colnames(result$opt.theta)
		result$opt.rho		= result$rho[result$opt.index]
	}

	if( sum(result$opt.adj ) < p - 2 )
	{
		ind <- unlist(lapply(1:length(result$rho), function(i) sum(result$adj[[i]])))
		result$opt.index	= which(ind >  ncol(result$data))[1] + 1 
		if( (is.na(result$opt.index)) || (result$opt.index > length(result$rho))) result$opt.index = length(result$rho)
		result$opt.theta	= result$Theta[[result$opt.index]]
		colnames(result$opt.theta) = colnames(result$data)
		result$opt.adj		= abs(sign(result$opt.theta)) - diag(rep(1,p))
		colnames(result$opt.adj) = colnames(result$opt.theta)
		result$opt.rho		= result$rho[result$opt.index]
	}
	
		if(verbose)
		{
			cat("done.\n")
			flush.console()
		}
	result <- list(opt.adj=result$opt.adj, opt.theta=result$opt.theta, opt.rho=result$opt.rho, opt.index=result$opt.index, theta=result$Theta, path= result$path, ES=result$ES, Z=result$Z, rho=result$rho, loglik=result$loglik, data=result$data )
	return(result)
}

npn = function(x, npn.func = "shrinkage"){
	gcinfo(FALSE)
	n = nrow(x)
  	p = ncol(x)
  	x.col = colnames(x)
  	x.row = rownames(x)
  	
	if(npn.func == "shrinkage"){
		
		x = qnorm(apply(x,2,rank)/(n+1))
		x = x/sd(x[,1])
   		colnames(x) = x.col
		rownames(x) = x.row
	}
	
	if(npn.func == "skeptic"){
		x = 2*sin(pi/6*cor(x,method="spearman"))
  		colnames(x) = x.col
		rownames(x) = x.col
	}
	return(x)
}

#-------------------------------------------------------------------------------#
# Package: Network-Based Genome-Wide Association Studies                        #
# Author: Pariya Behrouzi                                                       #
# Emails: <pariya.Behrouzi@gmail.com>                                           #
# Date: Nov 21th 2017                                                           #
#-------------------------------------------------------------------------------#

element_S <- function( lower, upper, mu=0, sigma=1)
{
  	delta1 <- (lower - mu) / sigma
  	delta2 <- (upper - mu) / sigma
  	tmp1 <- (dnorm(delta1) - dnorm(delta2)) / (pnorm(delta2) - pnorm(delta1))
  	EX <- mu + tmp1 * sigma
  	
  	delta1[delta1 < -1e+10] <- -1e+10
  	delta2[delta2 > 1e+10] <- 1e+10
  	tmp2 <- (delta1*dnorm(delta1) - delta2*dnorm(delta2)) / (pnorm(delta2) - pnorm(delta1))
  	EXX <- sigma^2 + mu^2 + sigma^2 * tmp2 + 2*mu*sigma*tmp1
	rm(delta1, delta2, tmp1, tmp2 )
	gc()
	
	return(list(EX=EX, EXX=EXX))
}

#-------------------------------------------------------------------------------#
# Package: Network-Based Genome-Wide Association Studies                        #
# simRIL(): generates genotype data from recombinant inbred lines population    #
# Author: Pariya Behrouzi                                                       #
# Emails: <pariya.Behrouzi@gmail.com>                                           #
# Date: Nov 21th 2017                                                           #
# Version: 0.0.1-1                                                              #
#-------------------------------------------------------------------------------#
simRIL <- function( d = 25, n = 200, g = 5, cM = 100, selfing=2 )
{
	Mar		<- NULL
	position<- NULL
	pos_chrs<- NULL 
	LG		<- NULL
	clusters<- NULL 
	orders	<- NULL 
	orders2 <- NULL 
	
	for ( j in 1: g )
	{
		LG 		 <-c(LG, rep(j, d ))
		clusters <-c(clusters, rep(j, d - 1))
		Mar		 <-c(Mar, 1:d )
		orders	 <-c(orders, 1:(d - 1) )
		orders2  <- c(orders2, 2:d )
		cM_dist  <- seq(0, cM, by = cM/(d - 1))
		pos_chrs <- c(pos_chrs, diff(cM_dist))
		position <- c(position, cM_dist)
	}
	rfr = 1/2* (1 - exp(-2 * pos_chrs / 100))
	
	F1 <- matrix(c(rep(0, d * g), rep(1, d*g) ) ,ncol=2, nrow= d * g) 
	sim_F1 <- vector("list", n); sim_F1 <- lapply(sim_F1, function(x) F1)	
	map <- data.frame(LG, paste("M", Mar, ".", LG, sep=""), position)
	colnames(map) <- c("chr", "marker", "cM")
	
	neighbor_mr <- data.frame(clusters, paste("M", orders, ".", clusters, sep=""), paste("M", orders2, ".", clusters, sep=""), pos_chrs, rfr)
	colnames(neighbor_mr) <- c("chr", "start", "end" , "pos_chrs", "recom")
	for (s in 1:selfing )
	{
		for (i in 1:length(sim_F1))
		{
			A <- NULL
			B <- NULL
			gam <- NULL
			haplo <- sim_F1[[i]]

						
			for (chr in 1:g)
			{
				neighbor_mr2 <- neighbor_mr[which(neighbor_mr[ ,"chr"] == chr), ]
				haplo2		 <- haplo[which(map[, "chr"] == chr), ]						
				A_new <- make_gamete( haplo2 = haplo2, neighbor_mr2 = neighbor_mr2, state = "A")	
				B_new <- make_gamete( haplo2 = haplo2, neighbor_mr2 = neighbor_mr2, state = "B")
				
				A <- c(A, A_new)	
				B <- c(B, B_new)		
			}
									
			sim_F1[i] <- list(cbind(A, B))
		}	
	}
		
	dat <- NULL
	for (i in 1:length(sim_F1))
	{
		haplo <- sim_F1[[i]] 
		dat <- cbind(dat, haplo[ ,1]+ haplo[ ,2] )
	}
	colnames(dat) <- paste("ind", 1:n, sep="")
	rownames(dat) <- map[ ,"marker"]	
	sim <- list(data= t(dat), map = map)	
	return(sim)
}	
	
make_gamete <- function( haplo2, neighbor_mr2, state = "A" )
{
	pr <- rbinom(1, 1, 0.5) + 1
	gameteA <- haplo2[ ,pr]
	gameteB <- haplo2[ ,setdiff(c(1,2), pr)]
	gam <- gameteA
	for( m in 1: nrow(neighbor_mr2))
	{	
		cr_over <- rbinom(1, 1, neighbor_mr2[m, "recom"]) 
		if (cr_over == 1 & state == "B"){gam[m+1] <- gameteA[m+1]; state2 <- "A"}
		if (cr_over == 0 & state == "B"){gam[m+1] <- gameteB[m+1]; state2 <- "B"}
		if (cr_over == 0 & state == "A"){gam[m+1] <- gameteA[m+1]; state2 <- "A"}
		if (cr_over == 1 & state == "A"){gam[m+1] <- gameteB[m+1]; state2 <- "B"}

		state <- state2
	}								
	return(gam)
}		

#-------------------------------------------------------------------------------#
# Package: Network-Based Genome-Wide Association Studies                        #
# Author: Pariya Behrouzi                                                       #
# Emails: <pariya.Behrouzi@gmail.com>                                           #
# Date: Nov 21th 2017                                                           #
#-------------------------------------------------------------------------------#

element_S <- function( lower, upper, mu=0, sigma=1)
{
  	delta1 <- (lower - mu) / sigma
  	delta2 <- (upper - mu) / sigma
  	tmp1 <- (dnorm(delta1) - dnorm(delta2)) / (pnorm(delta2) - pnorm(delta1))
  	EX <- mu + tmp1 * sigma
  	
  	delta1[delta1 < -1e+10] <- -1e+10
  	delta2[delta2 > 1e+10] <- 1e+10
  	tmp2 <- (delta1*dnorm(delta1) - delta2*dnorm(delta2)) / (pnorm(delta2) - pnorm(delta1))
  	EXX <- sigma^2 + mu^2 + sigma^2 * tmp2 + 2*mu*sigma*tmp1
	rm(delta1, delta2, tmp1, tmp2 )
	gc()
	
	return(list(EX=EX, EXX=EXX))
}

cond_N <- function(j, Sigma, Z , Z_new, diag_element, lower_upper)
{
	p <- ncol(Sigma)
	tmp <- matrix(Sigma[j, -j], 1, p-1)
	tmp1 <- solve(Sigma[-j, -j])

	mu <- tmp %*% tmp1 %*% t(Z[, -j])				
	mu <- as.vector(mu)		 
	sigma <- Sigma[j, j] - tmp %*% tmp1 %*% t(tmp)
	sigma <- sqrt(sigma)  		
		  
	obj <- element_S( lower= lower_upper$lower[ ,j], upper= lower_upper$upper[ ,j], mu=mu, sigma=sigma)      

	Z_new <- obj$EX
	diag_element <- mean(obj$EXX)
	
	rm(tmp, tmp1, mu, sigma, obj )
	gc()
	
	return(list(Z_new=Z_new, diag_element=diag_element))
}

calculate.R.approx.internal <- function(y, Z, lower_upper, Sigma, ncores = NULL )
{      
	if(is.null(ncores)) ncores = detectCores() - 1
	p <- ncol(y)
	n <- nrow(y) 
	Z_new <- rep(0, n)
	
	if(ncores > 1){
		cl <- makeCluster(ncores)
		Sigma <- Sigma
		Z <- Z
		cond_norm <- parLapply(cl = cl, 1:p, function(j) { 
			cond_N(j, Sigma=Sigma, Z=Z , Z_new=Z_new, diag_element=diag_element, lower_upper=lower_upper ); 
		})
		stopCluster(cl)
	}else{
		cond_norm <- lapply(1:p, function(j){ cond_N(j, Sigma, Z , Z_new, diag_element, lower_upper); } )
	}
	
	Z_new <- do.call(cbind, lapply(1:p, function(x) cond_norm[[x]]$Z_new ))
	diag_element <- sapply(1:p, function(x) cond_norm[[x]]$diag_element)
	
	ES <- t(Z_new) %*% Z_new / n
	diag(ES) <- diag_element

	output <- list()
	output$ES <- ES
	output$Z <- Z_new
	
	rm(lower_upper, ES, Z_new )
	return(output)         
}


R.approx = function(y, Z = NULL, Sigma=NULL, rho = NULL ,  ncores = NULL )
{
	p <- ncol(y)
	n <- nrow(y) 
	if(is.null(ncores)) ncores= detectCores()
	lower.upper <- lower.upper(y)
	
	if( is.null(Z) )
	{
		## Initialize S
		Z <- matrix(0, n, p)
		diag_element <- rep(0, p)
		if(ncores > 1)
		{
			cl <- makeCluster(ncores)
			tmp2 <- parLapply(cl = cl, 1:p, function(i) { 
			element_S_j(i, lower.upper ); 
			})
			stopCluster(cl)
		}else{
			tmp2 <- lapply(1:p, function(i){ element_S_j(i, lower.upper );})
		}
		Z <-  do.call(cbind, lapply(1:p, function(x) tmp2[[x]]$EX ))
		diag_element <- unlist(lapply(1:p, function(x)mean(tmp2[[x]]$EXX)))
		ES <- t(Z) %*% Z / n
		diag(ES) <- diag_element
		rm(tmp2, diag_element)	 
		gc() 
	}
	
	if(is.null(Sigma))
	{
		if(is.null(rho)) rho = max(max(ES),-min(ES))
		obj <- glasso(s=ES, rho=rho, maxit=1000, penalize.diagonal=FALSE)
		Sigma <- (t(obj$w) + obj$w) / 2
		sd_marginal  <- sqrt(diag(Sigma))
		sd_marginal[ abs(sd_marginal) < 1e-10 ] <- 1e-10
		Sigma <- diag(1/sd_marginal) %*% Sigma %*% diag(1/sd_marginal)
	}
	
	calculate.R.approx.internal( y, Z, lower.upper, Sigma, ncores = ncores)
}

#-------------------------------------------------------------------------------#
# Package: Network-Based Genome-Wide Association Studies                        #
# buildMap(): build a one-dimensional map from netgwasmap object    	      		#
# Author: Pariya Behrouzi                                                       #
# Emails: <pariya.Behrouzi@gmail.com>                                           #
# Date: Nov 21th 2017                                                           #
# Version: 0.0.1-1                                                              #
#-------------------------------------------------------------------------------#
buildMap.internal = function( network, cross, num.iso.m, use.comu )
{
	id <- colnames(network)
	p <- ncol(network)
	D <- diag(network)
	network[upper.tri(network)] <- 0
	network <- network + t(network)
	diag(network) <- D
	colnames(network) <- id
	
	# LG
	if(is.null(num.iso.m)) num.iso.m = floor(p/100)
	if( num.iso.m == 0 ) num.iso.m <- 1
	if(cross == "inbred")theta <- No.trans(as.matrix(network)) else theta <- as.matrix(network)
	memberships		<- community(theta, use.comu) 
	singl.elem <- which(lapply(memberships, function(Y) length(Y) ) <= num.iso.m)
	if(length(singl.elem) != 0) memberships <- memberships[- singl.elem ]
	rm(singl.elem)

	#ordering
	if(cross == "inbred")
	{
		cond.cor <- matrix(NA, ncol=p, nrow=p)
		for(i in 1:p)
		{
			for(j in 1:p)
			{
				cond.cor[i,j] <- - theta[i,j]/ sqrt(theta[i,i]* theta[j,j])
			}
		}
		sigma 	<- - solve(cond.cor)
		cr		<- cov2cor(sigma) 
		colnames(cr) <- colnames(theta)
		rownames(cr) <- colnames(theta)
		MDS <- lapply(1:length(memberships), function(Y) MDS.order(LG=Y, cr=cr, theta=theta, memberships=memberships ))
		map <- NULL
		k   <- 1
		while( k <= length(MDS)){
		  map <- rbind( map, MDS[[k]]$map )
		  k <- k + 1
		}
			        
		rm(MDS)     
	}
	
	if(cross == "outbred")
	{
		ord <- vector("list", length(memberships))
		th.map <- vector("list", length(memberships) )
		LG <- vector("list", length(memberships) )
		for(lg in 1: length(memberships))
			{
				index <- match( memberships[[lg]], id)
				th.LG <- network[index ,index]
				rownames(th.LG) <- colnames(th.LG)
				adj <- as.matrix(abs(sign(th.LG)) - diag(ncol(th.LG)))
				if(length(unique(adj[upper.tri(adj)])) == 1) adj[1,2] <- adj[2,1] <- 0  #full graph
			
				cuthill <- cuthill_mckee(adj)
				ord[[lg]] <- colnames(cuthill)
				ind <- match(colnames(cuthill), colnames(th.LG))
				#adj <- graphAM(adj)
				#cuthill <- cuthill.mckee.ordering(adj)
				#ord[[lg]] <- cuthill$`reverse cuthill.mckee.ordering`
				#ind <- match(cuthill$`reverse cuthill.mckee.ordering`, colnames(th.LG))
				th.map[[lg]]  <- th.LG[ind, ind]
				LG[[lg]] <- rep(lg, length(ind))
			}
		map <- cbind(markers= unlist(ord), LG=unlist(LG))
	}
	return(map)
}


buildMap = function(res, opt.index, min.m = NULL, use.comu = FALSE){
	
	#if(class(res) == "netgwasmap")
  if( inherits(res, "netgwasmap") )
	{
		if( is.null(opt.index)) stop("opt.index needs to be specified! \n")
		if( is.null(use.comu)) use.comu <- FALSE
		p <- ncol(res$allres$Theta[[opt.index]])
		if(is.null(min.m)) min.m = floor(p/100)
		if( min.m == 0 ) min.m <- 3
		cross <- res$cross.typ
		
		if(cross == "inbred") theta <- No.trans(as.matrix(res$allres$Theta[[opt.index]])) else theta <- as.matrix(res$allres$Theta[[opt.index]])
		D <- diag(theta)
		id <- colnames(theta)
		theta[upper.tri(theta)] <- 0
		theta <- theta + t(theta)
		diag(theta) <- D
		colnames(theta) <- id 
		memberships		<- community(theta, use.comu)
		singl.elem <- which(lapply(memberships, function(Y) length(Y) ) <= min.m)
		if(length(singl.elem) != 0) memberships <- memberships[- singl.elem ]
		rm(D, id)	
	
		if(cross == "inbred")
		{
			cond.cor <- matrix(NA, ncol=p, nrow=p)
			for(i in 1:p)
			{
				for(j in 1:p)
				{
					cond.cor[i,j] <- - theta[i,j]/ sqrt(theta[i,i]* theta[j,j])
				}
			}
			sigma 	<- - solve(cond.cor)
			cr		<- cov2cor(sigma) 
			colnames(cr) <- colnames(theta)
			rownames(cr) <- colnames(theta)
			MDS <- lapply(1:length(memberships), function(Y) MDS.order(LG=Y, cr=cr, theta=theta, memberships=memberships ))
			map <- NULL
			k   <- 1
			while( k <= length(MDS)){
			  map <- rbind( map, MDS[[k]]$map )
			  k <- k + 1
			}
		}
		if(cross == "outbred")
		{
			ord <- vector("list", length(memberships) )
			th.map <- vector("list", length(memberships))
			LG <- vector("list", length(memberships) )
			for(lg in 1: length(memberships))
			{
				index <- match( memberships[[lg]],colnames(theta))
				th.LG <- theta[index ,index]
				rownames(th.LG) <- colnames(th.LG)
				adj <- as.matrix(abs(sign(th.LG)) - diag(ncol(th.LG)))
				if(length(unique(adj[upper.tri(adj)])) == 1) adj[1,2] <- adj[2,1] <- 0
                cuthill <- cuthill_mckee(adj)
                ord[[lg]] <- colnames(cuthill)
				ind <- match(colnames(cuthill), colnames(th.LG))
        #adj <- graphAM(adj)
				#cuthill <- RBGL :: cuthill.mckee.ordering(adj)
				#ord[[lg]] <- cuthill$`reverse cuthill.mckee.ordering`
				#ind <- match(cuthill$`reverse cuthill.mckee.ordering`, colnames(th.LG))
				th.map[[lg]]  <- th.LG[ind, ind]
				LG[[lg]] <- rep(lg, length(ind))
			}
			map <- cbind(markers= unlist(ord), LG= unlist(LG))
		}
		results <- list(map= map, opt.index= opt.index, cross= cross, allres= res$allres, man= TRUE)
		class(results$allres) = "netgwas"
		class(results) = "netgwasmap"
		return(results)
	}else{
		stop("netgwasmap.object should belong to the netgwasmap class. \n ")
	}
}

#' Cuthill McKee (CM) algorithm
#' 
#' Transform sparse matrix into a band matrix
#' netprioR ::: cuthill_mckee()
#' @author Fabian Schmich
#' @import Matrix
#' @param x Input matrix
#' @return Band matrix
cuthill_mckee <- function(x) {
  degs <- data.frame(Idx=1:ncol(x), NonZero=apply(x, 1, function(x) length(which(x != 0))))
  R <- degs$Idx[which.min(degs$NonZero)]
  i <- 1
  for (i in 1:ncol(x)) {
    Ai <- setdiff(which(x[R[i],] != 0), R)
    if (length(Ai) > 0) {
      Ai <- Ai[order(degs$NonZero[Ai], decreasing = FALSE)]
      R <- append(R, Ai)
    } else {
      R <- append(R, degs$Idx[-R][which.min(degs$NonZero[-R])])
    }
    i <- i + 1
  }
  rR <- rev(R)
  return(x[rR, rR])
}


#### Check for invariant and markers#####
cleaning.dat = function(dat){
	dat <- as.matrix(dat, byrow= FALSE, ncol= ncol(dat), nrow= nrow(dat) )
	#remove fixed-value (invariant) markers
	fixd.val <-  which(apply(dat, 2, function(x) length(unique(sort(x)))) == 1)
	if(length(fixd.val) == 0){dat <- dat }else {dat <- dat[ ,- fixd.val ]}
	return(dat)
}

No.trans <- function(theta){
  dig <- diag(theta)
  theta[theta > 0 ] <-  0
  diag(theta) <- dig
  return(theta)
}

##### community detection
community = function(theta, use.comu){
	p		<- ncol(theta)
	path	<- abs(sign(theta)) - diag(rep(1,p))
	colnames(path) <- colnames(theta)
	adj <- graph.adjacency(path, mode="undirected")
	if(use.comu == TRUE)
	{
		fg <- cluster_fast_greedy(adj)
		memberships <- lapply(1:length(fg), function(x) fg[[x]]) 
	}else{
		cl  <- clusters(adj)
		grp <- groups(cl)
		memberships <- lapply(1:cl$no, function(x)  grp[[x]] ) 
	}
	return(memberships)
}


##### Ordering markers per LG 
MDS.order <- function( LG=3, cr, theta, memberships )
{
  index <- match(memberships[[LG]],colnames(cr))
  cr.LG <- cr[index ,index]
  theta.LG <- theta[index ,index]
  rownames(cr.LG) <- colnames(cr.LG)
  rownames(theta.LG) <- colnames(theta.LG)
  cr.LG[cr.LG <= 0] <- runif(1, 1e-08, 5e-08)
  dis		<- -log(cr.LG)
  dis.1	<- sammon(dis, k = 1, trace =FALSE)
  MDS.path <- order(dis.1$points)
  MDS.tour <- colnames(cr.LG )[MDS.path]
  chr <- rep(LG, length(MDS.tour))
  ordered <- data.frame(markers=MDS.tour, LG=chr )
  theta.LG <- theta.LG[MDS.path , MDS.path ]
  cr.LG <- cr.LG[MDS.path , MDS.path ]
  results <- list(map=ordered, cr.LG=cr.LG, theta.LG=theta.LG )
 
  return(results)
}

##### constructors graphAM as a subclasses of graph
graphAM <- function (adjMat = matrix(integer(), 0, 0), edgemode='undirected', values=NA)
{
    if (length (values) == 1L && is.na (values))
        new("graphAM", adjMat=adjMat, edgemode=edgemode)
    else
        new("graphAM", adjMat=adjMat, edgemode=edgemode,
                 values=values)
}

#-----------------------------------------------------------------------#
# Package: Network-Based Genome-Wide Association Studies                #
# simgeno(): Generate an ordinal data with genome-like graph structure  #
# Author: Pariya Behrouzi                                               #
# Emails: <pariya.Behrouzi@gmail.com>                                   #
# Date: Nov 21th 2017                                                   #
# Version: 0.0.1-1                                                      #
#-----------------------------------------------------------------------#

simgeno = function( p = 90, n = 200, k = NULL, g = NULL, adjacent = NULL, alpha = NULL, beta = NULL, con.dist = "Mnorm", d = NULL, vis = TRUE)
{

  if(is.null(k)) k = 3
  if(is.null(g))
  {
  	g = 1
  	if(p > 40)	g = ceiling(p/20)
  	if(p <= 40) g = 2
  }

	g.large = p %% g
	g.small = g - g.large
	n.small = floor(p/g)
	n.large = n.small+1
	g.list = c(rep(n.small,g.small),rep(n.large,g.large))
	g.ind = rep(c(1:g), g.list)
	rm(g.large, g.small, n.small, n.large)
	gc()
	
	A = matrix(0, p, p);	
	g.count = c(0, cumsum(g.list))
	if(is.null(adjacent)) adjacent = 1 
	for(i in 1:adjacent){
	for(j in 1: (length(g.count)-1)){
		diag(A[(g.count[j]+1):(g.count[j+1] - i), ((g.count[j]+1)+i):g.count[j +1 ]]) = 1
		diag(A[((g.count[j]+1)+i):g.count[j+1], (g.count[j]+1):(g.count[j+1] - 1) ]) = 1
	}
	}
	diag(A) <- 1
	image(Matrix(A))
	
	if(is.null(alpha)) alpha <- 0.01
	alpha = 1 - alpha
	for(i in 1:g){
		tmp = which(g.ind==i)
		tmp2 = matrix(runif(length(tmp)^2, 0, 0.5),length(tmp),length(tmp))
		tmp2 = tmp2 + t(tmp2)		 	
		A[tmp,tmp][tmp2 > alpha] = 1  
		rm(tmp,tmp2)
		gc()
	}
	image(Matrix(A))
	
	if(is.null(beta)) beta <- 0.02
	beta = 1 - beta
	tmp2 = matrix(runif(p^2, 0, 0.5), p, p)
	tmp2 = tmp2 + t(tmp2)
	A[tmp2 > beta] = 1
	rm(tmp2)
	image(Matrix(A))
	
	u= 0.1
	V <- matrix( 0, p, p )
	diag(A) = 0
	V[upper.tri(V)] <- runif(p*(p-1)/2, min=0.5, max=0.8)
	V <- V + t(V)
	theta = A * - V
	diag(theta) = abs(min(eigen(theta)$values)) + 0.1 + u
	sigma = cov2cor(solve(theta))
		
	if(is.null(con.dist)) con.dist = "Mnorm"	
	if(con.dist == "Mt")
	{
		if(is.null(d)) d = 3
		z = rmvt(n=n, sigma = sigma, df = d)
		sigmahat = cor(z)	
	}
  
	if(con.dist == "Mnorm")
	{
		z = rmvnorm(n=n, sigma=sigma)		
		sigmahat = cor(z)	
	}

	cutoffs   <- matrix( runif( k * p ), nrow = p, ncol = k )   
	marginals <- apply( cutoffs, 1, function(x) { qnorm( cumsum( x / sum(x) )[-length(x)] ) } )
  
	for ( j in 1:p )
	{
		breaks <- c( min( z[,j] ) - 1, marginals[,j], max(z[,j]) + 1 )  
		z[,j]  <- as.integer( cut( z[,j], breaks = breaks, right = FALSE ) )
	}	
	if ( vis == TRUE )
	{
		Figuers = par(mfrow = c(1, 2), pty = "s", omi=c(0.3,0.3,0.3,0.3), mai = c(0.3,0.3,0.3,0.3))
	 
		adj <- graph.adjacency( A, mode = "undirected", diag=FALSE )
		colnames(A)   <- paste("M", 1:p, sep="")
		if ( p < 50 ) label = colnames(A) else label = NA 
		if ( p < 50 ) size = 15 else size = 7
		color         <-  terrain.colors(g)
		ly <- layout_with_fr(adj)
		Figuers[1] = plot(adj, layout = ly , vertex.color = color[g.ind], vertex.size =size, vertex.label = label, vertex.label.color = 'black', vertex.label.dist=0, main = "Graph structure")
		Figuers[2] = image(A, col = gray.colors(2, start=1, end=0), xaxt="n", yaxt='n', main = "Adjacency Matrix")
		title(ylab = "markers", cex.lab = 1, line = .5)
		title(xlab = "markers", cex.lab = 1, line = .5)
		box()
		rm(Figuers)
		gc()
    }
	z <- z - 1
	
	colnames(z) <- colnames(A)
	simulation <- list( data=z, Theta=theta, adj=Matrix(A, sparse=TRUE), Sigma=sigma, n.groups=g, groups=g.ind, sparsity= sum(A)/(p*(p-1)))
  
	class(simulation) = "simgeno"
	return( simulation )
}


#-----------------------------------------------------#
#   		Plot for class "simgeno"        	      #
#-----------------------------------------------------#
plot.simgeno = function(x, layout = layout.fruchterman.reingold, ...){

   	par = par(mfrow = c(2, 2), pty = "s", omi=c(0.3,0.3,0.3,0.3), mai = c(0.3,0.3,0.3,0.3))
   	if ( ncol(x$data) < 50 ) size = 10 else size = 7
	if(is.null(layout)) layout = layout.fruchterman.reingold
	color =  terrain.colors(x$n.groups)
	g = graph.adjacency(x$adj, mode="undirected", diag=FALSE)
	plot(g, layout=layout, vertex.color=color[x$groups], vertex.size = size, vertex.label = NA,  vertex.label.dist=0, main = "Graph structure")
	
	
	image(as.matrix(x$adj), col = gray.colors(2, start=1, end=0), xaxt="n", yaxt="n", main = "Adjacency matrix")
	title(ylab = "markers", cex.lab = 1, line = .5)
	title(xlab = "markers", cex.lab = 1, line = .5)
	box()
	image(as.matrix(x$Theta), col = gray.colors(300), main = "Precision matrix")
	image(as.matrix(x$Sigma), col = gray.colors(300), main = "Covariance matrix")
	
	rm(g, layout, color)
	gc()
	
}

#-----------------------------------------------------#
#   		Summary for class "simgeno"        		  #
#-----------------------------------------------------#
print.simgeno = function(x, ...){
	cat("Simulated data generated by generate.ord()\n")
	cat("Number of variables: p =", ncol(x$data), "\n")
	cat("Sample size: n =", nrow(x$data), "\n")
	cat("Number of levels: k =", length(unique(sort((x$data)))), "\n")
    cat("Number of groups: g =", x$n.groups, "\n")
	cat("Graph type = genome-like", "\n")
    cat("Sparsity level:", sum(x$Theta)/ncol(x$data)/(ncol(x$data)-1),"\n")
}

############################################################################
#################   Detecting Genotyping error markers  ####################
###########################################################################

detect.err <- function(netgwas.map,  err.prob = 0.01, cutoff= 4, pop.type= NULL, map.func = "haldane" ){
  
  if(is.null(pop.type))  stop("Population type needs to be \"BC\",\"DH\",\"ARIL\" or \"RILn\" (see ?netgwas.as.cross).")
  
  map <- cal.pos(netgwas.map, pop.type= pop.type , map.func = map.func )
  err <- calc.errorlod(map, error.prob= err.prob)
  top.errorlod(err, cutoff= cutoff)
  
}

#---------------------------------------------------------------------------------#
#From mvtnorm package: https://cran.r-project.org/web/packages/mvtnorm/index.html #
#---------------------------------------------------------------------------------#

isInf <- function(x) x > 0 & is.infinite(x)

rmvnorm = function(n, mu=NULL, sigma )
{
	if(is.null(mu)) mu = rep(0, n)
	p = nrow(sigma)
	y  <- matrix(rnorm(n*p),nrow = p, ncol= n)
	chl<- chol(sigma)
	z <- t( t(chl) %*% y + mu )
return(z)
}

dmvnorm <- function (x, mean = rep(0, p), sigma = diag(p), log = FALSE)
{
	p = ncol(x)
	if(is.null(mean)) mean = rep(0, p)
    if (is.vector(x))
	x <- matrix(x, ncol = length(x))
    p <- ncol(x)
    if(!missing(mean)) {
	if(!is.null(dim(mean))) dim(mean) <- NULL
	if (length(mean) != p)
	    stop("mean and sigma have non-conforming size")
    }
    if(!missing(sigma)) {
	if (p != ncol(sigma))
	    stop("x and sigma have non-conforming size")
	if (!isSymmetric(sigma, tol = sqrt(.Machine$double.eps),
			 check.attributes = FALSE))
	    stop("sigma must be a symmetric matrix")
    }

    
    dec <- tryCatch(chol(sigma), error=function(e)e)
    if (inherits(dec, "error")) {
        x.is.mu <- colSums(t(x) != mean) == 0
        logretval <- rep.int(-Inf, nrow(x))
        logretval[x.is.mu] <- Inf 
    } else {
	tmp <- backsolve(dec, t(x) - mean, transpose = TRUE)
	rss <- colSums(tmp ^ 2)
	logretval <- - sum(log(diag(dec))) - 0.5 * p * log(2 * pi) - 0.5 * rss
    }
    names(logretval) <- rownames(x)
    if(log) logretval else exp(logretval)
}


rmvt <- function(n, sigma = diag(2), df = 1,
                 delta = rep(0, nrow(sigma)),
                 type = c("shifted", "Kshirsagar"))
{
    if (length(delta) != nrow(sigma))
        stop("delta and sigma have non-conforming size")

    if (df == 0 || isInf(df))
	{
		return(rmvnorm(n, mu = delta, sigma = sigma))
	}	
    type <- match.arg(type)
    switch(type,
           "Kshirsagar" = {
               return(rmvnorm(n, mu = delta, sigma = sigma)/
                      sqrt(rchisq(n, df)/df))
           },
           "shifted" = {
               sims <- rmvnorm(n, sigma = sigma)/sqrt(rchisq(n, df)/df)
               return(sweep(sims, 2, delta, "+"))
           },
           stop("wrong 'type'"))
}

#-------------------------------------------------------------------------------#
# Package: Network-Based Genome-Wide Association Studies                        #
# Author: Pariya Behrouzi                                                       #
# Emails: <pariya.Behrouzi@gmail.com>                                           #
# Date: Nov 21th 2017                                                           #
#-------------------------------------------------------------------------------#

cond_N <- function(j, Sigma, Z , Z_new, diag_element, lower_upper)
{
	p <- ncol(Sigma)
	tmp <- matrix(Sigma[j, -j], 1, p-1)
	tmp1 <- solve(Sigma[-j, -j])

	mu <- tmp %*% tmp1 %*% t(Z[, -j])				
	mu <- as.vector(mu)		 
	sigma <- Sigma[j, j] - tmp %*% tmp1 %*% t(tmp)
	sigma <- sqrt(sigma)  		
		  
	obj <- element_S( lower= lower_upper$lower[ ,j], upper= lower_upper$upper[ ,j], mu=mu, sigma=sigma)      

	Z_new <- obj$EX
	diag_element <- mean(obj$EXX)
	
	rm(tmp, tmp1, mu, sigma, obj )
	gc()
	
	return(list(Z_new=Z_new, diag_element=diag_element))
}

#-------------------------------------------------------------------------------#
# Package: Network-Based Genome-Wide Association Studies                        #
# Author: Pariya Behrouzi                                                       #
# Emails: <pariya.Behrouzi@gmail.com>                                           #
# Date: Nov 21th 2017                                                           #
#-------------------------------------------------------------------------------#

calculate_EM_approx = function(chain, y, Z, rho, Sigma, Theta, lower_upper, em_tol, em_iter, ncores=4, verbose=FALSE )
{
	c_em_iter = 1
	dif	<- 100
	p 	<- ncol(y)
	n	<- nrow(y)
	s <- proc.time()	
	while((c_em_iter < em_iter) && (dif >= em_tol ))
	{
		S_obj	<- calculate.R.approx.internal(y=y, Z=Z, lower_upper=lower_upper, Sigma= Sigma, ncores=ncores)
		Z		<- S_obj$Z
		S_gl	<- glasso(s=S_obj$ES, rho=rho, maxit=1000, penalize.diagonal=FALSE)	
		Theta <- (t(S_gl$wi) + S_gl$wi) / 2
		Sigma_new <- (t(S_gl$w) + S_gl$w) / 2
		sd_marginal <- sqrt(diag(Sigma_new))
		sd_marginal[abs(sd_marginal) < 1e-10] <- 1e-10			
		Sigma_new	<- diag(1/sd_marginal) %*% Sigma_new %*% diag(1/sd_marginal)
		Theta	<- Matrix(diag(sd_marginal) %*% Theta %*% diag(sd_marginal))	
		dif		<-	sum(abs(Sigma_new - Sigma)/p^2) 
		Sigma	<-  Sigma_new
		c_em_iter <- c_em_iter + 1
		rm(Sigma_new, sd_marginal, S_gl)
		gc()
	}

	results <- list()
	results$Theta	<- Theta
	results$Sigma	<- Sigma
	results$ES		<- S_obj$ES
	results$Z		<- Z
	results$rho		<- rho
	results$loglik	<- n/2 *(determinant(results$Theta, logarithm = TRUE)$modulus - sum(diag(results$ES %*%results$Theta)))

	return(results)
}

#-------------------------------------------------------------------------------#
# Package: Network-Based Genome-Wide Association Studies                        #
# Author: Pariya Behrouzi                                                       #
# Emails: <pariya.Behrouzi@gmail.com>                                           #
# Date: Nov 21th 2017                                                           #
#-------------------------------------------------------------------------------#

initialize = function(y, rho = NULL, n_rho = NULL, rho_ratio = NULL, ncores=NULL )
{
	p <- ncol(y)
	n <- nrow(y)
	lower_upper = lower.upper(y)
	
	if(is.null(rho))
	{
		if(is.null(n_rho)) n_rho = 10
		if(is.null(rho_ratio)) rho_ratio = 0.3
		cr = cor(y, method="spearman") - diag(p)
		cr[is.na(cr)] <- 0
		rho_max = max( max(cr),-min(cr) )
		if(rho_max == 0) 
		{
			ty <- npn(y, npn.func= "shrinkage")
			cr = cor(ty, method="spearman") - diag(p)
			rho_max = max(max(cr),-min(cr))
		}
		if(rho_max >= .7) rho_max = .7
		rho_min = rho_ratio * rho_max
		rho = exp(seq(log(rho_max), log(rho_min), length = n_rho))
		rm(cr, rho_max, rho_min, rho_ratio)
		gc()
	}

	## Initialize S
	Z <- matrix(0, n, p)
	diag_element <- rep(0, p)
	if(ncores > 1)
	{
		cl <- makeCluster(ncores)
		tmp2 <- parLapply(cl = cl, 1:p, function(i) { 
		element_S_j(i, lower_upper ); 
		})
		stopCluster(cl)
	}else{
		tmp2 <- lapply(1:p, function(i){ element_S_j(i, lower_upper );})
	}
	Z <-  do.call(cbind, lapply(1:p, function(x) tmp2[[x]]$EX ))
	diag_element <- unlist(lapply(1:p, function(x)mean(tmp2[[x]]$EXX)))
	ES <- t(Z) %*% Z / n
	diag(ES) <- diag_element
	rm(tmp2, diag_element)	 
	gc() 

	return(list(Z=Z, ES = ES, rho = rho, lower_upper=lower_upper))
}	

#-------------------------------------------------------------------------------#
# Package: Network-Based Genome-Wide Association Studies                        #
# selectnet(): Select optimal network using three methods: Bayesian information #
#criterion(BIC), extended BIC (eBIC), and Akaike information criterion (AIC)    #
# Author: Pariya Behrouzi                                                       #
# Email: <pariya.Behrouzi@gmail.com>                                            #
# Date: Nov 21th 2017                                                           #
# Version: 0.0.1-1                                                              #
#-------------------------------------------------------------------------------#

selectnet = function(netgwas.obj, opt.index = NULL, criteria = NULL, ebic.gamma = 0.5, ncores = NULL, verbose=TRUE)
{
	if(! is.null(opt.index)){
		if(is.null(ncores)) ncores <- 1
		sel <- list( )
		sel$opt.adj		<-  netgwas.obj$path[[opt.index]]
		sel$opt.theta	<- netgwas.obj$Theta[[opt.index]]
		sel$opt.rho		<- netgwas.obj$rho[opt.index]
		sel$opt.index	<- opt.index
	}else{
		if(is.null(ncores)) ncores <- detectCores() - 1
		if(is.null(criteria)) criteria <- "ebic"
		if(!ncores) ncores = 1
		sel	= model.selection( netgwas.obj, criterion = criteria, lower.upper=lower.upper, ebic.gamma=ebic.gamma, ncores = ncores, verbose = verbose)
	}

	theta <- as.matrix(sel$opt.theta)
	rownames(theta) <- colnames(theta)
	par.cor <- calculate.strength.theta(theta)
	par.cor[upper.tri(par.cor)] <- 0
	par.cor <- par.cor + t(par.cor)
	diag(par.cor ) <- 1
	par.cor[par.cor > 1] <- 1
	sel$par.cor <- Matrix(par.cor)

	A <- graph.adjacency(sel$opt.adj, mode = "undirected")
	sel$V.names <- names (which (degree(A) != 0 ))


	rm(par.cor, theta, netgwas.obj)
	class(sel) = "select"
	return(sel)
}

readkey <- function()
{
    cat ("Press [enter] to continue")
    line <- readline()
}

#-----------------------------------------------------#
#different plots for class "select"                   #
# Author: Pariya Behrouzi                             #
# Email: <pariya.Behrouzi@gmail.com>                  #
#-----------------------------------------------------#
plot.select = function(x, vis= NULL, xlab= NULL, ylab= NULL, n.mem = NULL, vertex.label = FALSE, ..., layout = NULL, label.vertex = "all", vertex.size = NULL, vertex.color = NULL , edge.color = "gray29", sel.nod.label = NULL, label.size = NULL, w.btw= 800, w.within = 10,
                  sign.edg= TRUE, edge.width= NULL, edge.label= NULL,  max.degree= NULL, layout.tree= NULL, root.node= NULL, degree.node= NULL, curve= FALSE, delet.v =TRUE, pos.legend= "bottomleft", cex.legend= 0.8, iterl = NULL, temp = NULL, tk.width = NULL, tk.height= NULL)
  #plot.select = function(x, vis= NULL, xlab= NULL, ylab= NULL, n.mem = NULL, vertex.label = FALSE, ..., layout = NULL, label.vertex = "all", vertex.size = NULL, vertex.color = "red" , edge.color = "gray29", sel.nod.label = NULL, label.size = NULL, w.btw= 800, w.within = 10)
{
	if(! inherits( x, "select" )) stop("The input of this plot function should be from \"select\" class (More info in: selectnet( ) ). \n")
	if(is.null(vis)) vis <-  "CI"
	if(is.null(xlab)) xlab <- "variables"
	if(is.null(ylab)) ylab <- "variables"
	plot.new()
	par(mfrow= c(1,1))

	if(delet.v == TRUE)
	  {
  	  A <- graph.adjacency(x$opt.adj, mode = "undirected")
  	  A <- delete.vertices(A, degree(A) == 0)
  	  x$opt.adj <- Matrix(as_adj(A))
   	  x$par.cor   <- Matrix(x$par.cor[colnames(x$opt.adj) ,colnames(x$opt.adj) ])
   	  rownames(x$opt.theta) <- colnames(x$opt.theta)
  	  x$opt.theta <- Matrix(x$opt.theta [colnames(x$opt.adj) ,colnames(x$opt.adj)] )
	  }else{
	  cat(" isolated nodes are shown in the graph \n ")
	}

	if(vis == "image.parcorMatrix") print(image(Matrix(x$par.cor), xlab=xlab, ylab=ylab, main= "partial correlation matrix", sub="") )
	if(vis == "image.adj") print(image(Matrix(x$opt.adj), xlab=xlab, ylab=ylab, main= "adjacency matrix", sub=""))
	if(vis == "image.precision") print(image(Matrix(x$opt.theta), xlab=xlab, ylab=ylab, main= "precision matrix", sub=""))

	if(vis == "CI" ){

		if(! vertex.label) {
			vertex.label = NA
		}else{
			if(!is.null(colnames(x$opt.adj) ))
				{
					vertex.label = colnames(x$opt.adj)
			}else{
					vertex.label= NA
			}
		}
		if(is.null(vertex.size)) vertex.size <- 7
		if(is.null(cex.legend)) cex.legend <- 0.8

		adj = graph.adjacency(as.matrix(x$opt.adj), mode="undirected", diag=FALSE)
		if(is.null(n.mem))
			{
				memberships = 1
				vertex.color = "red"
		}else{
			LG = length(n.mem)
			memberships = NULL
			i = 1
			while( i <= LG)
				{
					grp <- rep(i, n.mem[i])
					memberships = c(memberships, grp)
					i = i + 1
				}
			color <- sample(rainbow(max(memberships)+10, alpha=0.3), max(memberships))
			vertex.color = color[memberships]
			names(memberships) <- colnames(x$opt.adj) #membership network
			E(adj)$weight=apply(get.edgelist(adj), 1, weight.community,memberships, w.btw, w.within)#membership network
		}

		if(is.null(n.mem)){
			layout	= layout.fruchterman.reingold
		}else{
			layout = layout.fruchterman.reingold(adj, weights=E(adj)$weight) #membership network
		}

		plot(adj, layout= layout, vertex.color= vertex.color , edge.color='gray40', vertex.size = vertex.size, vertex.label = vertex.label, vertex.label.dist = 0, main= "Selected graph")
		if(length(memberships) > 1) legend(pos.legend, paste("group", 1:length(n.mem)), cex=cex.legend, col= color, pch=rep(20,10))
		readkey()

		#if(is.null(xlab)) xlab <- ""
		#if(is.null(ylab)) ylab <- ""
		image(as.matrix(x$opt.adj), xaxt="n", yaxt="n", col = gray.colors(256) ,main="Conditional dependence relationships" , cex=0.8)
		title(ylab = ylab, cex.lab = 1, line = .5)
		title(xlab = xlab, cex.lab = 1, line = .5)
	}

	# correspondence to the plotG3.R (my local) file.
	if(vis == "parcor.network"){
	  if(is.null(edge.width)) edge.width = FALSE
	  if(is.null(edge.label)) edge.label = FALSE
	  if(is.null(label.size)) label.size <-  0.9
	  if(is.null(vertex.size)) vertex.size <- 5
	  if(is.null(layout.tree)) layout.tree = FALSE
	  if(is.null(root.node)) root.node <- 1
	  if(is.null(degree.node)) degree.node <- 0
	  if(is.null(cex.legend)) cex.legend <- 0.8
	  if(is.null(vertex.color)) vertex.color <- "lightblue3"

	  p <- ncol(x$par.cor)
	  par.cor <- as.matrix(x$par.cor)

	  if(sign.edg == TRUE)
	  {
	    adj <- graph.adjacency(par.cor, weighted=TRUE, diag=FALSE, mode= "lower")
	    E(adj)$color[(E(adj)$weight < 0) ] <-  'darkblue'
	    E(adj)$color[(E(adj)$weight > 0) ] <-  'red3'
	    E(adj)$lty[abs(E(adj)$weight) >= 0.90 ] <- 1
	    E(adj)$lty[(abs(E(adj)$weight) < 0.90) & ( abs(E(adj)$weight) >= 0.65 )] <- 6
	    E(adj)$lty[(abs(E(adj)$weight) < 0.65) & (abs(E(adj)$weight) >= 0.35 )] <- 5
	    E(adj)$lty[(abs(E(adj)$weight) < 0.35) & ( abs(E(adj)$weight) >= 0.10 )] <- 2
	    E(adj)$lty[ (abs(E(adj)$weight) < 0.10) & ( abs(E(adj)$weight) >= 0.00 )] <- 3
	  }else{
	    path <-  abs(sign(par.cor)) - diag(rep(1,p))
	    adj <- graph.adjacency(path, mode="undirected")
	    E(adj)$color <- "gray40"
	  }
	  V(adj)$label.cex <- label.size
	  V(adj)$label <- colnames(par.cor)

	  if(layout.tree == TRUE)
	  {
	    pathA <-  abs(sign(par.cor)) - diag(rep(1,p))
	    A <- graph.adjacency(pathA, mode="undirected")
	    layout <- layout_as_tree(A, root = root.node)
	  }else{
	    pathA <- abs(sign(par.cor)) - diag(rep(1,p))
	    A <- graph.adjacency(pathA, mode="undirected")
	    #if(is.null(layout)) layout <- layout_with_fr(A)
	    if(is.null(layout)) layout <- layout_with_kk(A)
	  }

	  if(layout.tree == TRUE){
	    if(is.null(degree.node)) {
	      deg <- 0
	      vertex.label.dist <- 0
	    }
	    if(!is.null(degree.node)) {
	      deg <- degree.node
	      vertex.label.dist <- 1
	    }
	  }else{
	    deg <- degree.node
	    vertex.label.dist <- 0
	  }
	  if(edge.label == TRUE) edge.label=round(E(adj)$weight,2) else edge.label= NULL
	  if(edge.width == TRUE) edge.width = E(adj)$weight else edge.width= NULL

	  if((!is.null(max.degree)) && (is.null(sel.nod.label)) ) plot(adj, vertex.label=ifelse(degree(adj) >= max.degree, V(adj)$label, NA), layout=layout, edge.curved = curve,  vertex.color=vertex.color,
	                                                               vertex.size=vertex.size, layout = layout, vertex.label.color="black", vertex.label.degree =deg , label.degree= deg, vertex.label.dist= vertex.label.dist)
	  if((!is.null(sel.nod.label)) && (is.null(max.degree))) plot(adj, edge.label= edge.label, edge.width= edge.width, vertex.label=ifelse(V(adj)$label %in% sel.nod.label, V(adj)$label, NA ), layout=layout, edge.curved = curve,  vertex.color=vertex.color,
	                                                              vertex.size=vertex.size, layout = layout, vertex.label.color="black",  vertex.label.degree =deg, label.degree= deg, vertex.label.dist= vertex.label.dist)
	  if((is.null(max.degree) ) && (is.null(sel.nod.label))) plot(adj, vertex.label= colnames(par.cor), layout=layout, edge.curved = curve,  vertex.color=vertex.color,
	                                                              vertex.size=vertex.size, layout = layout, vertex.label.color="black", vertex.label.degree =deg, label.degree= deg, vertex.label.dist= vertex.label.dist)

	  if(! is.null(E(adj)$lty)) legend(pos.legend, legend=c( ">= 0.90", "0.90-0.65" , "0.65-0.35", "0.35-0.10", "0.10-0.00") , col="black", cex=cex.legend,  lty= c(1, 6, 5, 2, 3), title=" |partial corr|" )

	}

	if(is.null(tk.width))  tk.width <- 1000
	if(is.null(tk.height)) tk.height <- 1000
	if(vis == "interactive"){
		adj <- as.matrix(x$opt.adj)

		if(is.null(vertex.size)) vertex.size <- 7
		if(is.null(label.vertex)) label.vertex <- "all"
		if(is.null(vertex.color)) vertex.color <- "red"
		if(is.null(edge.color)) edge.color <- "gray29"
		if(is.null(sel.nod.label)) sel.nod.label <- NULL
		if((label.vertex == "some") && (is.null(sel.nod.label )) ) stop("Please select some vertex label(s) or fix label.vertex to either none or all.")

		p <- ncol(adj)
		A <- graph.adjacency(adj, mode= "undirected")

				if(is.null(n.mem))
			{
				memberships = 1
				#vertex.color = "red"
		}else{
			LG = length(n.mem)
			memberships = NULL
			i = 1
			while( i <= LG)
				{
					grp <- rep(i, n.mem[i])
					memberships = c(memberships, grp)
					i = i + 1
				}
			#color <- sample(rainbow(max(memberships)+10, alpha=0.3), max(memberships))
			#vertex.color = color[memberships]
			names(memberships) <- colnames(x$opt.adj) #membership network
			E(A)$weight=apply(get.edgelist(A), 1, weight.community,memberships, weigth.within= w.btw, weight.between= w.within )#membership network
		}

		if(is.null(n.mem)){
			#if(is.null(layout)) layout <- layout_with_fr(A)
			layout	= layout.fruchterman.reingold
		}else{
			layout = layout.fruchterman.reingold(A, weights=E(A)$weight) #membership network
		}

		if(is.null(layout)) layout <- layout_with_fr(A)
		if(is.null(label.size)) label.size <- 1
		V(A)$label.cex <- label.size

		if(label.vertex == "none")
		{
			V(A)$label <- NA
			tkplot(A, layout=layout, vertex.color=vertex.color, edge.color=edge.color, vertex.size=vertex.size, vertex.label.dist=0, canvas.width = tk.width, canvas.height = tk.height)
		}

		if(label.vertex == "some")
		{
			V(A)$label <- colnames(adj)
			tkplot(A, vertex.label=ifelse(V(A)$label %in% sel.nod.label, V(A)$label, NA ), layout=layout, vertex.color=vertex.color, edge.color=edge.color, vertex.size=vertex.size, vertex.label.dist=0, canvas.width = tk.width, canvas.height = tk.height)
		}
		if(label.vertex == "all")
		{
			V(A)$label <- colnames(adj)
			tkplot(A, vertex.label=colnames(adj) , layout=layout, vertex.color=vertex.color, edge.color=edge.color, vertex.size=vertex.size, vertex.label.dist=0, canvas.width = tk.width, canvas.height = tk.height)
		}
	}

	if(vis == "parcor.interactive"){
	  adj <- as.matrix(x$par.cor)
	  #%adj <- as.matrix(x$opt.adj)

	  if(is.null(vertex.size)) vertex.size <- 7
	  if(is.null(label.vertex)) label.vertex <- "all"
	  if(is.null(vertex.color)) vertex.color <- "red"
	  #if(is.null(edge.color)) edge.color <- "gray29"
	  if(is.null(sel.nod.label)) sel.nod.label <- NULL
	  if((label.vertex == "some") && (is.null(sel.nod.label )) ) stop("Please select some vertex label(s) or fix label.vertex to either none or all.")

	  p <- ncol(adj)
	  A <- graph.adjacency(adj, weighted=TRUE, diag=FALSE, mode= "lower")

	  E(A)$color[(E(A)$weight < 0) ] <-  'darkblue'
	  E(A)$color[(E(A)$weight > 0) ] <-  'red3'

	  if(is.null(n.mem))
	  {
	    memberships = 1

	  }else{
	    LG = length(n.mem)
	    memberships = NULL
	    i = 1
	    while( i <= LG)
	    {
	      grp <- rep(i, n.mem[i])
	      memberships = c(memberships, grp)
	      i = i + 1
	    }

	    names(memberships) <- colnames(x$opt.adj)
	    E(A)$weight=apply(get.edgelist(A), 1, weight.community,memberships, weigth.within= w.btw, weight.between= w.within )#membership network
	  }

	  if(is.null(n.mem)){
	    layout	<- layout.fruchterman.reingold
	  }else{
	    #layout = layout.fruchterman.reingold(A, weights=E(A)$weight) #membership network
	    layout <- layout_with_kk(A)
	  }
	  if(is.null(temp)) temp <- sqrt(vcount(A))
	  if(is.null(iterl)) iterl <- 500
	  if(is.null(layout)) layout <- layout_with_fr(A, coords = NULL, dim = 2, niter = iterl, start.temp = temp,  weights= E(A)$weight)
	  if(is.null(label.size)) label.size <- 1
	  V(A)$label.cex <- label.size

	  if(label.vertex == "none")
	  {
	    V(A)$label <- NA
	    tk <- tkplot(A, layout=layout, vertex.color=vertex.color, edge.color=edge.color, vertex.size=vertex.size, vertex.label.dist=0, canvas.width = tk.width, canvas.height = tk.height)
	  }

	  if(label.vertex == "some")
	  {
	    V(A)$label <- colnames(adj)
	    tk <- tkplot(A, vertex.label=ifelse(V(A)$label %in% sel.nod.label, V(A)$label, NA ), layout=layout, vertex.color=vertex.color, vertex.size=vertex.size, vertex.label.dist=0, canvas.width = tk.width, canvas.height = tk.height)
	  }
	  if(label.vertex == "all")
	  {
	    V(A)$label <- colnames(adj)
	    tk <- tkplot(A, vertex.label=colnames(adj) , layout=layout, vertex.color=vertex.color, edge.color=edge.color, vertex.size=vertex.size, vertex.label.dist=0 , canvas.width = tk.width, canvas.height = tk.height)
	  }
	  #return(tkplot.getcoords(tk))
	}
}

calculate.strength.theta <- function(theta){
  p <- ncol(theta)
  cond.cor <- matrix(NA, ncol=p, nrow=p)
  for(i in 1:nrow(theta))
  {
    for(j in 1:ncol(theta))
    {
      cond.cor[i,j] <- - (theta[i,j])/ (sqrt(theta[i,i])* sqrt(theta[j,j]))
    }
  }
  id <- colnames(theta)
  rownames(cond.cor) <- id
  colnames(cond.cor) <- id
  return(cond.cor)
}


#-----------------------------------------------------#
#   		Summary for class "select"                #
#-----------------------------------------------------#
print.select = function(x, ...){
	cat("To plot selected graph: plot(<YOUR OUTPUT NAME>) \n")
	cat("To visualize an interactive network: plot(<YOUR OUTPUT NAME>, vis= \"interactive\") \n")
	cat("To plot partial correlations: image(<YOUR OUTPUT NAME>$par.cor) \n")
}


weight.community=function(row,membership,weigth.within,weight.between){
  if(as.numeric(membership[which(names(membership)==row[1])])==as.numeric(membership[which(names(membership)==row[2])])){
    weight=weigth.within
  }else{
    weight=weight.between
  }
  return(weight)
}

context("netphenogeno")

test_that("npn in netphenogeno works fine ", 
  {
	 data(thaliana)
	 L.M1 <- netphenogeno(thaliana[ 1:5, c(1:5, 61:65)], method="npn", rho = 0.3)$loglik
	
	  expect_equal( ceiling(L.M1), -6 )  
   })


test_that("Gibbs in netphenogeno works fine ", 
  {
	  data(CviCol)
	  L.M2 <- netphenogeno(thaliana[ 1:5, c(1:5, 61:65)], method="gibbs", rho = 0.5, ncores = 1)$loglik
	
	  expect_equal( ceiling(L.M2) , -15 )  
   })

\name{netphenogeno}
\alias{netphenogeno}

\title{Reconstructs conditional dependence network among genetic loci and phenotypes}

\description{
This is one of the main functions of the \pkg{netgwas} package. This function reconstructs a conditional independence network between genotypes and phenotypes for diploids and polyploids. Three methods are available to reconstruct networks, namely (i) Gibbs sampling, (ii) approximation method, and (iii) nonparanormal approach within the Gaussian copula graphical model. The first two methods are able to deal with missing genotypes. The last one is computationally faster.
}

\usage{
netphenogeno(data, method = "gibbs", rho = NULL, n.rho = NULL, rho.ratio = NULL,
		ncores = 1, em.iter = 5, em.tol=.001, verbose = TRUE)
}

\arguments{
	\item{data}{ An (\eqn{n \times p}) matrix or a \code{data.frame} corresponding to the data matrix (\eqn{n} is the sample size and \eqn{p} is the number of variables). The \eqn{p} columns include either a marker or trait(s) information. Input data can contain missing values.}
	\item{method}{Reconstructing both genotype-phenotype interactions network and genotype-phenotype-environment interactions network with three methods: "gibbs", "approx", and "npn". For a medium (~500) and a large number of variables we recommend to choose "gibbs" and "approx", respectively. Choosing "npn" for a very large number of variables (> 2000) is computationally efficient. The default method is "gibbs".}
	\item{rho}{ A decreasing sequence of non-negative numbers that control the sparsity level. Leaving the input as \code{rho = NULL}, the program automatically computes a sequence of \code{rho} based on \code{n.rho} and \code{rho.ratio}. Users can also supply a decreasing sequence values to override this.}
	\item{n.rho}{ The number of regularization parameters. The default value is \code{10}.}
	\item{rho.ratio}{ Determines distance between the elements of \code{rho} sequence. A small value of \code{rho.ratio} results in a large distance between the elements of \code{rho} sequence. And a large value of \code{rho.ratio} results into a small distance between elements of \code{rho}. The default value is \code{0.3}.}
	\item{ncores}{ The number of cores to use for the calculations. Using \code{ncores = "all"} automatically detects number of available cores and runs the computations in parallel on (available cores - 1).}
	\item{em.iter}{ The number of EM iterations. The default value is 5.}
	\item{em.tol}{ A criteria to stop the EM iterations. The default value is .001. }
	\item{verbose}{Providing a detail message for tracing output. The default value is \code{TRUE}.}
}

\details{
This function reconstructs both genotype-phenotype network and genotype-phenotype-environment interactions network. In genotype-phenotype networks nodes are either markers or phenotypes; each phenotype is connected by an edge to a marker if there is a direct association between them given the rest of the variables. Different phenotypes may also interconnect.  In addition to markers and phenotypes information, the input data can include environmental variables. Then, the interactions network shows the conditional dependence relationships between markers, phenotypes and environmental factors.}

\value{
An object with S3 class \code{"netgwas"} is returned:  
 \item{Theta}{ A list of estimated p by p precision matrices that show the conditional independence relationships patterns among measured items.}
 \item{path}{ A list of estimated p by p adjacency matrices. This is the graph path corresponding to \code{Theta}.}
 \item{ES}{ A list of estimated p by p conditional expectation corresponding to \code{rho}.}
 \item{Z}{ A list of n by p transformed data based on Gaussian copula. }
 \item{rho}{ A \code{n.rho} dimensional vector containing the penalty terms.}
 \item{loglik }{ A \code{n.rho} dimensional vector containing the maximized log-likelihood values along the graph path. }
 \item{data}{ The \eqn{n} by \eqn{p} input data matrix. The \eqn{n} by \eqn{p} transformed data in case of using "npn". }
}

\note{This function estimates a graph path . To select an optimal graph please refer to \code{\link{selectnet}}.}

\author{ 
  Pariya Behrouzi and Ernst C. Wit \cr
  Maintainers: Pariya Behrouzi \email{pariya.behrouzi@gmail.com}\cr
}

\references{
1. Behrouzi, P., and Wit, E. C. (2019). Detecting epistatic selection with partially observed genotype data by using copula graphical models. Journal of the Royal Statistical Society: Series C (Applied Statistics), 68(1), 141-160. \cr
2. Behrouzi, P., and Wit, E. C. (2017c). netgwas: An R Package for Network-Based Genome-Wide Association Studies. arXiv preprint, arXiv:1710.01236. \cr
3. D. Witten and J. Friedman. New insights and faster computations for the graphical lasso. \emph{Journal of Computational and Graphical Statistics}, to appear, 2011. \cr
4. Guo, Jian, et al. "Graphical models for ordinal data." Journal of Computational and Graphical Statistics 24.1 (2015): 183-204. \cr
}

\seealso{\code{\link{selectnet}}}

\examples{
    \dontshow{
		data(thaliana)
		D <- thaliana[1:5, 1:10]
		res <-  netphenogeno(data= D, n.rho=2, em.iter = 1, ncores=1)
       }
    \donttest{
		data(thaliana)
		head(thaliana, n=3)
		#Construct a path for genotype-phenotype interactions network in thaliana data
		res <-  netphenogeno(data = thaliana); res
		plot(res)
		#Select an optimal network
		sel <- selectnet(res)
		#Plot selected network and the conditional correlation (CI) relationships 
		plot(sel, vis="CI")
		plot(sel, vis="CI", n.mem = c(8, 56, 31, 33, 31, 30), w.btw =50, w.within= 1)
		
		#Visualize interactive plot for the selected network
		#Color "red" for 8 phenotypes, and different colors for each chromosome.
		cl <- c(rep("red", 8), rep("white",56), rep("tan1",31), 
		      rep("gray",33), rep("lightblue2",31), rep("salmon2",30))
		      
		#The IDs of phenotypes and SNPs to be shown in the network       
    id <- c("DTF_LD","CLN_LD","RLN_LD","TLN_LD","DTF_SD","CLN_SD","RLN_SD", 
        "TLN_SD","snp15","snp16","snp17","snp49","snp50","snp60","snp75",
        "snp76","snp81","snp83","snp84","snp86","snp82", "snp113","snp150",
        "snp155","snp159","snp156","snp161","snp158","snp160","snp162","snp181")
		
		plot(sel, vis="interactive", n.mem = c(8, 56, 31, 33, 31, 30), vertex.color= cl,
		    label.vertex= "some", sel.nod.label= id, edge.color= "gray", w.btw= 50,
		    w.within= 1)
		
		#Partial correlations between genotypes and phenotypes in the thaliana dataset.
		library(Matrix)
		image(sel$par.cor, xlab="geno-pheno", ylab="geno-pheno", sub="")
	}
}

\name{netgwas2cross}
\alias{netgwas2cross}

\title{ netgwasmap object to cross object}

\description{
Convertes \code{netgwasmap} object from  \code{net.map} or \code{buildMap} functions to \code{cross} object from R/\pkg{qtl} package.
}

\usage{
   netgwas2cross(netgwasmap, pop.type= NULL, map.func = "haldane")
}

\arguments{
  \item{netgwasmap}{A \code{netgwasmap} object. The output of \code{\link{netmap}} or \code{\link{buildMap}} functions.}
  \item{pop.type}{Character string specifying the population type of the genotype data. Accepted values are "DH" (doubled haploid), "BC" (backcross), "RILn" (non-advanced RIL population with n generations of selfing) and "ARIL" (advanced RIL).}
  \item{map.func}{Character string defining the distance function used for calculation of genetic distances. Options are "kosambi", "haldane", and "morgan". Default is "haldane".}
}

\details{
  If \code{pop.typ = "RILn"} the number of generations of selfing is limited to 20 to ensure sensible input. The constructed object is returned as a R/qtl \code{cross} object with the appropriate class structure. For \code{"RILn"} populations the constructed object is given the class \code{"bcsft"} by using the \pkg{qtl} package conversion function \code{convert2bcsft} with arguments \code{F.gen = n} and \code{BC.gen = 0}. For \code{"ARIL"} populations the constructed object is given the class \code{"riself"}.
  
  In R/\pkg{qtl} package, the genotype data for a backcross is coded as NA = missing, 1 = AA, 2 = AB. For an F2 intercross, the coding is NA = missing, 1 = AA, 2 = AB, 3 = BB, 4 = not BB (i.e. AA or AB), 5 = not AA (i.e. AB or BB).
}

\value{
  The \code{netgwas.map} object is returned as a \code{cross} object form R/\pkg{qtl}. The object is a list with usual components \code{"pheno"} and \code{"geno"}.
  
  \item{geno}{The \code{"geno"} element contains \code{data} and \code{map} for separated linkage groups which have been constructed using \code{net.map} function. }
  \item{pheno}{Character string containing the genotype names.}
}

\author{ 
  Pariya Behrouzi \cr
  Maintainer: Pariya Behrouzi \email{pariya.behrouzi@gmail.com}\cr
}

\examples{
  \dontrun{
    sim <- simRIL(d=25, n=200, g=5, cM=100, selfing= 2)
    # to use the same genotyping coding as R/qtl package (See details)
    sim$data <- (sim$data) + 1 
    
    #Estimate linkage groups and order markers within each LG
    out <- netmap(sim$data, cross = "inbred")
    map <- out$map; map
    
    plot(out)
    
    #Calculate map positions and convert the map to cross object from qtl package
    map <- netgwas2cross(netgwasmap = out, pop.type= "RIL2", map.func = "haldane" )
    plotMap(map)
  }
}

\name{netgwas-internal}
\alias{netgwas-internal}
\title{netgwas-internal}

\description{
  Internal netgwas functions. These are not intended to be called by the users.
}

\author{Pariya Behrouzi <pariya.behrouzi@gmail.com> and Ernst C. Wit}

\keyword{internal}

\name{cal.pos}
\alias{cal.pos}

\title{Estimate genetic map distances }

\description{
Calculation of genetic map distances for an estimated markers order from either \code{net.map} or \code{buildMap} functions. This function is only for diploid populations. We note that the output of \code{net.map} and \code{buildMap} functions include estimated linkage groups and estimated markers order within each linkage group. 
}

\usage{
cal.pos (netgwasmap, pop.type= NULL , map.func = "haldane", chr )
}

\arguments{
	\item{netgwasmap}{A \code{netgwasmap} object. The output of \code{\link{netmap}} or \code{\link{buildMap}} functions.}
	\item{pop.type}{Character string specifying the population type of the genotype data. Accepted values are "DH" (doubled haploid), "BC" (backcross), "RILn" (non-advanced RIL population with n generations of selfing) and "ARIL" (advanced RIL) (see Details). }
	\item{map.func}{Character string defining the distance function used for calculation of genetic
distances. Options are "kosambi", "haldane", and "morgan". Default is "haldane".}
	\item{chr}{A character string of linkage group names that require calculating of their genetic map distances.}
	}

\details{
 In \pkg{qtl} package, the genotype data for a backcross is coded as NA = missing, 1 = AA, 2 = AB. For an F2 intercross, the coding is NA = missing, 1 = AA, 2 = AB, 3 = BB, 4 = not BB (i.e. AA or AB), 5 = not AA (i.e. AB or BB).
 
If \code{pop.typ = "RILn"} the number of generations of selfing is limited to 20 to ensure sensible input. The constructed object is returned as a R/qtl \code{cross} object with the appropriate class structure. For \code{"RILn"} populations the constructed object is given the class \code{"bcsft"} by using the \pkg{qtl} package conversion function \code{convert2bcsft} with arguments \code{F.gen = n} and \code{BC.gen = 0}. For \code{"ARIL"} populations the constructed object is given the class \code{"riself"}.

This function uses the Viterbi algorithm implemented in \code{argmax.geno} of the \pkg{qtl} package to estimate genetic distances. Initial conservative estimates of the map distances are calculated from inverting recombination fractions outputted from \code{est.rf}. These are then passed to \code{argmax.geno} and imputation of missing allele scores is performed along with re-estimation of map distances. This is an adapted version of \code{quickEst} function from \pkg{ASMap} package.
}

\value{
The \code{netgwas} constructed linkage map is returned as a \code{R/qtl} cross object. The object is a list with usual components \code{"pheno"} and \code{"geno"}.

 \item{geno}{The \code{"geno"} element contains \code{data} and \code{map} for separated linkage groups which have been constructed using \code{net.map} function. }
 \item{pheno}{Character string containing the genotype names.}
}

\author{ 
  Pariya Behrouzi \cr
  Maintainer: Pariya Behrouzi \email{pariya.behrouzi@gmail.com}\cr
}

\examples{
\dontrun{
sim <- simRIL(d=25, n=200, g=5, cM=100, selfing= 2)
 # to use the same genotyping coding as qtl package (See details)
sim$data <- (sim$data) + 1 

 #Estimate linkage groups and order markers within each LG
out <- netmap(sim$data, cross = "inbred")
map <- out$map; map

plot(out)

 #Calculate map positions and convert the map to cross object from qtl package
pos.map <- cal.pos(netgwasmap = out, pop.type= "RIL2", map.func = "haldane" )
plotMap(pos.map)
}
}

\name{CviCol}
\alias{CviCol}
\docType{data}
\title{Arabidopsis thaliana genotype data }
\description{ 
  The genotype data of the Cvi-0 \eqn{\times} Col-0 Recombinant Inbred Line (RIL) population.
}

\usage{data(CviCol)}
\format{
  The format is a matrix containing 90 single-nucleotide polymorphism (SNP) markers for 367 individuals.
}  

\details{
The \emph{Arabidopsis thaliana} genotype data is derived from a RIL cross between Columbia-0 (Col-0) and the Cape Verde Island (Cvi-0), where 367 individuals were genotyped for 90 genetic markers. This is a diploid population with three possible genotpe states \code{(k = 3)}, where the genotypes coded as \code{0, 1, 2}, where 0 and 2 represent the homozygous genotypes and 1 defines the heterozygous genotype. \cr
This data set can be used to detect epistatic selection, short- and long- range linkage disequilibrium between 90 SNP markers. 
}

\source{
Simon, M., et al. "QTL mapping in five new large RIL populations of Arabidopsis thaliana genotyped with consensus SNP markers." Genetics 178 (2008): 2253-2264.
It is publicly available at http://publiclines.versailles.inra.fr/page/8
}

\author{ 
  Pariya Behrouzi and Ernst C. Wit \cr
  Maintainer: Pariya Behrouzi \email{pariya.behrouzi@gmail.com}\cr
}


\examples{
data(CviCol)
dim(CviCol)
head(CviCol, n=3)
}

\keyword{datasets}

\name{cross2netgwas}
\alias{cross2netgwas}

\title{\code{cross} object to \code{netgwas} data frame}

\description{Converts \code{cross} object from R/\pkg{qtl} package to \code{netgwas} dataframe

}

\usage{
  cross2netgwas (cross.obj)
}

\arguments{
  \item{cross.obj}{An object of class \code{cross}.}
}

\value{
An (\eqn{n \times p}) matrix corresponds to a genotype data matrix (\eqn{n} is the sample size and \eqn{p} is the number of variables). This matrix can be as an input data for \code{\link{netmap}}, and \code{\link{netsnp}} functions.
}

\author{ 
  Pariya Behrouzi \cr
  Maintainer: Pariya Behrouzi \email{pariya.behrouzi@gmail.com}\cr
}

\name{buildMap}
\alias{buildMap}

\title{ linkage group detection and ordering markers for class "netgwasmap"}

\description{
Implements different algorithms for detecting linkage groups and ordering markers in each linkage group.
}

\usage{
buildMap( res, opt.index, min.m = NULL, use.comu = FALSE)
}

\arguments{
	\item{res}{ An object with S3 class "netgwasmap"}
	\item{opt.index}{ An index of a desired regularization parameter.}
	\item{min.m}{Expected minimum number of markers in a chromosome. Optional}
	\item{use.comu}{Using community detection algorithm to detect linkage groups. Default is FALSE.}
}

\details{
This function determines linkage groups and order markers within each linkage group for class "netgwasmap".
}

\value{
An object with S3 class \code{"netgwasmap"} is returned:  
 \item{map}{ Constructed linkage map associated with \code{opt.index}.}
 \item{opt.index}{ The index of a desired 3-D map to construct linkage map.}
 \item{cross}{The specified cross type by user. }
 \item{allres}{ A list containing results for different regularization parameter. Belongs to class "netgwas". To visualize a path of different 3D maps consider function \code{\link{plot.netgwas}}. Note that the input data is reordered based on the estimated linkage map and is saved as data in this argument.}
 \item{man}{stays TRUE.}
}

\references{
1. Behrouzi, P., and Wit, E. C. (2018). De novo construction of polyploid linkage maps using discrete graphical models. Bioinformatics. \cr
2. Behrouzi, P., and Wit, E. C. (2017c). netgwas: An R Package for Network-Based Genome-Wide Association Studies. arXiv preprint, arXiv:1710.01236.
}

\author{ 
  Pariya Behrouzi and Ernst C.Wit \cr
  Maintainer: Pariya Behrouzi \email{pariya.behrouzi@gmail.com} \cr
}

\seealso{\code{\link{netmap}}}

\examples{
\dontrun{
data(CviCol)
#Randomly change the order of markers across the genome
cvicol <- CviCol[ ,sample(ncol(CviCol))]

#Constructing linkage map for Cvi x Col genotype data
out <- netmap(cvicol, cross= "inbred", ncores=1); out
plot(out)
map <- out$map; map

#Visualization of other networks
plot(out$allres)  
#Constructing a linkage map for 5th network
bm <- buildMap(out, opt.index=5); bm
plot(bm, vis= "summary")
#or
plot(bm, vis= "interactive", label.vertex="all")
}
}

\name{plot.select}
\alias{plot.select}

\title{ Plot function for S3 class "select" }

\description{
Plot the optimal graph by model selection
}

\usage{
\method{plot}{select}(x, vis= NULL, xlab= NULL, ylab= NULL, n.mem= NULL, vertex.label= FALSE
, ..., layout= NULL, label.vertex= "all", vertex.size= NULL, vertex.color= NULL,
edge.color= "gray29", sel.nod.label= NULL, label.size = NULL, w.btw= 800,
 w.within = 10, sign.edg= TRUE, edge.width= NULL, edge.label= NULL,
max.degree= NULL, layout.tree= NULL, root.node= NULL, degree.node= NULL,
curve= FALSE, delet.v =TRUE, pos.legend= "bottomleft", cex.legend= 0.8,
iterl = NULL, temp = NULL, tk.width = NULL, tk.height= NULL)
}

\arguments{
	\item{x}{An object with S3 class "select" }
	\item{vis}{Visualizing the results as a graph (network) or as a matrix. There are 4 options to visulize the selected graph: (i) "CI": plotting conditional independence (CI) relationships between variables, (ii) "interactive": plotting the conditional independence network, where opens a new windows with interactive graph drawing facility, and (iii) "parcor.network": plots the estimated graph based on partial correlation values. (iv) "parcor.interactive": plots the estimated graph based on partial correlation matrix with an interactive graph drawing facility. Default is "CI". \cr
Also, there are 3 options to visulaze the selected graph as a matrix: (i) \code{vis}= "image.parcorMatrix" plots the image of partial correlation matrix, (ii)  \code{vis} = "image.adj" draws the adjacency matrix (only presence and absence of links), (iii) \code{vis} = "image.precision" plots the selected precision matrix.}
	\item{xlab}{ONLY applicable when \code{vis = "CI"}, "image.parcorMatrix", "image.adj", or "image.precision".}
	\item{ylab}{ONLY applicable when \code{vis = "CI"}, "image.parcorMatrix", "image.adj", or "image.precision".}
	\item{n.mem}{A vector of memberships. For example, the CviCol dataset, which is provided in the package, contain 5 chromosomes which the total number of markers is \eqn{p = 90}, where the first 24 markers belong into chromosome 1, the next 14 markers into chromosome 2, ..., and chromosome 5 contains 19 markers. Thus, n.mem = c(24,14,17,16,19). If \code{n.mem = NULL} and \code{vis = "CI"} all vertices are coloured the same.}
	\item{vertex.label}{ONLY applicable when \code{vis}= "CI". Assign names to the vertices. Default is FALSE.}
	\item{\dots}{ONLY applicable when \code{vis}= "CI". System reserved (No specific usage)}
	\item{layout}{ONLY applicable when \code{vis}= "interactive" or "parcor.network". The layout specification. Some graph layouts examples: layout_with_fr, layout_in_circle, layout_as_tree, and layout.fruchterman.reingold. The default layout is layout_with_fr.}
	\item{label.vertex}{ONLY applicable when \code{vis}= "interactive". Assign names to the vertices. There are three options: "none", "some", "all". Specify "none" to omit vertex labels in the graph; using \code{label.vertex} = "some" you must provide a vector of vertex IDs or a single vertex ID to the \code{sel.label} argument, which you would like to be shown in the graph. Specify "all" to include all vertex labels in the graph. Default is "all".}
	\item{vertex.size}{ Optional. The size of vertices in the graph visualization. The default value is 7.}
	\item{vertex.color}{ONLY applicable when \code{vis}= "interactive" or "parcor.network". Optional vector (or a color name) giving the colors of the vertices. The default is "red" }
	\item{edge.color}{ONLY applicable when \code{vis}= "interactive". Optional. The default is "gray".}
	\item{sel.nod.label}{ONLY applicable when \code{vis}= "interactive" or "parcor.network". A vector of vertex IDs or a single vertex ID, which you would like to be shown in the graph. ONLY applicable when \code{label.vertex}="some".}
	\item{label.size}{ONLY applicable for \code{vis}= "interactive" or \code{vis}= "parcor.network". The font size of the vertex labels.}
	\item{w.btw}{Distance between nodes from different memberships of \code{n.mem} in layout.}
	\item{w.within}{Distance of nodes within one membership of \code{n.mem} in layout.}
	\item{sign.edg}{Optional. ONLY applicable when \code{vis}= "parcor.network". If \code{TRUE} then edges are colored as red and blue, where red stands for positive and blue negative partial correlation values. If \code{FASLE} all edeges are colored as gray. Default is TRUE.}
	\item{edge.width}{Optional. ONLY applicable when \code{vis}= "parcor.network". Based on the strength of partial correlation values, edges will shown with different line type. Default is FALSE.}
	\item{edge.label}{Optional. ONLY applicable when \code{vis}= "parcor.network". If TRUE then the partial correlation values will be shown on top of each edge. Default is FALSE.}
	\item{max.degree}{Optional. ONLY applicable when \code{vis}= "parcor.network". A number showing degree of a node. This can be used to print those vertex labels that the correspondence vertex have at least e.g. 1 degree.}
	\item{layout.tree}{Optional. ONLY applicable when \code{vis}= "parcor.network". If TRUE then it uses \code{layout_as_tree} from \code{igraph} package. Default is FALSE.}
	\item{root.node}{Optional. ONLY applicable when \code{vis}= "parcor.network". The index of the root vertex or root vertices. If this is a non-empty vector then the supplied vertex ids are used as the roots of the trees . If it is an empty vector, then the root vertices are automatically calculated based on topological sorting, performed with the opposite mode than the mode argument. After the vertices have been sorted, one is selected from each component.}
	\item{degree.node}{Optional. ONLY applicable when \code{vis}= "parcor.network". It is related to the vertex label degree. It controls the position of the labels with respect to the vertices. Value are for example -pi/2, 0, pi/2, pi sets above, to the right, below, to the left of a node, respectively.}
	\item{curve}{Optional. ONLY applicable when \code{vis}= "parcor.network". Edge curvature, range between 0 and 1 (FALSE sets it to 0, TRUE to 0.5). Default is FALSE.}
	\item{delet.v}{ Delete vertices with no edges. Default is "TRUE".}
	\item{pos.legend}{Applicable when \code{vis}= "parcor.network" or \code{vis}= "CI". The x and y co-ordinates to be used to position the legend. They can be specified by keywords like "topright", "topleft", and etc. Default is "bottomleft".}
	\item{cex.legend}{Applicable when \code{vis}= "parcor.network" or \code{vis}= "CI".}
\item{iterl}{Optional. ONLY applicable when \code{vis}= "parcor.interactive". integer scalar, the number of iterations to perform for layout_with_fr layout.}
\item{temp}{Optional. ONLY applicable when \code{vis}= "parcor.interactive". Real scalar, the start temperature for layout_with_fr layout.}
\item{tk.width}{Optional. The size of the drawing area of interactive plot.}
\item{tk.height}{Optional. The size of the drawing area of interactive plot.}
}

\value{
An object with S3 class \code{"select"} is returned:
 \item{network}{ Plot of a selected graph,  when \code{vis}= "CI".}
 \item{adjacency}{ Conditional independence (CI) relationships between variables, when \code{vis}= "CI"}
 \item{network}{Interactive plot of a selected graph with .eps format, when \code{vis}= "interactive"}
}

\author{
  Pariya Behrouzi \cr
  Maintainer: Pariya Behrouzi \email{pariya.behrouzi@gmail.com} \cr
}

\references{
Behrouzi, P., and Wit, E. C. (2017c). netgwas: An R Package for Network-Based Genome-Wide Association Studies. arXiv preprint, arXiv:1710.01236.
}

%\seealso{\code{\link{select}}}


\examples{
	\donttest{
		#simulate data
		data(CviCol)
		out <- netsnp(CviCol)
		sel <- selectnet(out)

		cl <- c(rep("palegoldenrod", 24), rep("white",14), rep("tan1",17),
            rep("gray",16), rep("lightblue2",19))
    plot(sel, vis= "parcor.network", sign.edg = TRUE, layout = NULL, vertex.color = cl)
	}
}

\name{plot.netgwasmap}
\alias{plot.netgwasmap}

\title{ plot for S3 class "netgwasmap" }

\description{
Plot the graph associated with constructed linkage map via function \code{\link{netmap}}.
}

\usage{
\method{plot}{netgwasmap}(x, vis= NULL, layout= NULL, vertex.size= NULL, label.vertex =
		"none", label.size= NULL, vertex.color= NULL, edge.color = "gray29",
		sel.ID = NULL, ... )
}

\arguments{
	\item{x}{An object from "netgwasmap" class.}
	\item{vis}{Visualizing in four options: (i) "summary" plots the related network, conditional dependence relationships between markers before and after ordering markers; (ii) "interactive" plots the associated network, where it opens a new windows with interactive graph drawing facility; (iii) "unordered markers" plots the conditional dependence relationships between markers before ordering markers; (iv) "ordered markers" plots conditional dependence relationships between markers after ordering markers. Default is "summary".}
	\item{layout}{The vertex placement algorithm which is according to \pkg{igraph} package. The default layout is Fruchterman-Reingold layout. Other possible layouts are, for example, layout_with_kk, circle, and Reingold-Tilford graph in \pkg{igraph} package. }
	\item{vertex.size}{ Optional integer to adjust vertex size in graph G. Default is 5.}
	\item{label.vertex}{Assign names to the vertices. There are three options: "none", "some", "all". (i) Specifying "none" omits vertex labels in the graph, (ii) using \code{label.vertex} = "some" you need to provide a vector of vertex IDs or a single vertex ID to the \code{sel.ID} argument, which you would like to be shown in the graph. \code{label.vertex} = "some" is only applicable for \code{vis} = "interactive", (iii) Specifying "all" includes all vertex labels in the graph. Default is "none".}
	\item{label.size}{Optional integer to adjust the size of node's label in graph G. Applicable when vertex.label is TRUE. Default is 0.8.}
	\item{vertex.color}{Optional integer vectors giving colors to the vertices. }
	\item{edge.color}{Optional integer vectors giving colors to edges. }
	\item{sel.ID}{ONLY applicable when \code{vis}= "interactive". A vector of vertex IDs or a single vertex ID, which you would like to be shown in the graph. ONLY applicable when \code{label.vertex}="some".}
	\item{\dots}{ONLY applicable when \code{vis}= "CI". System reserved (No specific usage)}
}

\author{ 
  Pariya Behrouzi and Ernst C. Wit \cr
  Maintainer: Pariya Behrouzi \email{pariya.behrouzi@gmail.com}
}

\references{
1. Behrouzi, P., and Wit, E. C. (2018). De novo construction of polyploid linkage maps using discrete graphical models. Bioinformatics. \cr
2. Behrouzi, P., and Wit, E. C. (2017c). netgwas: An R Package for Network-Based Genome-Wide Association Studies. arXiv preprint, arXiv:1710.01236.
}


\seealso{\code{\link{netmap}}, \code{\link{buildMap}}.}

\name{detect.err}
\alias{detect.err}

\title{Identiying likely genotyping error}

\description{ Calculates a LOD score for each genotype, measuring the evidence for genotyping errors. This uses \code{calc.errorlod} function from R/\pkg{qtl} package.}

\usage{
detect.err(netgwas.map,  err.prob= 0.01, cutoff= 4, 
          pop.type= NULL, map.func= "haldane")
}

\arguments{
	\item{netgwas.map}{An object of class \code{netgwasmap} object (The output of \code{\link{netmap}} or \code{\link{netmap}} functions).}
	\item{err.prob}{Assumed genotyping error rate used in the calculation of the penetrance Pr(observed
genotype | true genotype).}
	\item{cutoff}{Only those genotypes with error LOD scores above this cutoff will be listed.}
	\item{pop.type}{Character string specifying the population type of the genotype data. Accepted values are "DH" (doubled haploid), "BC" (backcross), "RILn" (non-advanced RIL population with n generations of selfing) and "ARIL" (advanced RIL) (see Details).}
	\item{map.func}{Character string defining the distance function used for calculation of genetic
distances. Options are "kosambi", "haldane", and "morgan". Default is "haldane".}
	}

\value{
A data.frame with 4 columns, whose rows correspond to the genotypes that are possibly in error.
The four columns give the chromosome number, individual number, marker name, and error LOD
score.
}

\examples{
\dontrun{
sim <- simRIL(d=25, n=200, g=5, cM=100, selfing= 2)
 # to use the same genotyping coding as R/qtl package (See details)
sim$data <- (sim$data) + 1 

 #Estimate linkage groups and order markers within each LG
out <- netmap(sim$data, cross = "inbred")
map <- out$map; map
plot(out)

# A list of genotyoing error
detect.err(out, pop.type = "RIL2")
}
}

\name{print.select}
\alias{print.select}
\title{
Print function for S3 class "select"
}
\description{
Print function for \code{\link{selectnet}}.
}
\usage{
\method{print}{select}(x, ...)
}
\arguments{
  \item{x}{
An object with S3 class "select"
}
  \item{\dots}{
System reserved (No specific usage)
}
}

\author{
  Pariya Behrouzi and Ernst C. Wit \cr
  Maintainer: Pariya Behrouzi \email{pariya.behrouzi@gmail.com} \cr
}

\references{
Behrouzi, P., and Wit, E. C. (2017c). netgwas: An R Package for Network-Based Genome-Wide Association Studies. arXiv preprint, arXiv:1710.01236.
}

\seealso{
\code{\link{selectnet}}
}

\name{print.netgwas}
\alias{print.netgwas}
\title{
Print function for S3 class "netgwas"
}
\description{
Print a summary of results from function \code{\link{netsnp}}, \code{\link{netphenogeno}}. 
}
\usage{
\method{print}{netgwas}(x, ...)
}
\arguments{
  \item{x}{
An object with S3 class "netgwas"
}
  \item{\dots}{
System reserved (No specific usage)
}
}

\author{ 
  Pariya Behrouzi and Ernst C. Wit \cr
  Maintainer: Pariya Behrouzi \email{pariya.behrouzi@gmail.com} \cr
}

\references{
Behrouzi, P., and Wit, E. C. (2017c). netgwas: An R Package for Network-Based Genome-Wide Association Studies. arXiv preprint, arXiv:1710.01236.
}

\seealso{
\code{\link{netmap}}, \code{\link{netsnp}}, \code{\link{netphenogeno}}
}

\name{netgwas-package}
\alias{netgwas-package}
\docType{package}
\title{
  Network Based Genome Wide Association Studies
}

\description{
The \code{R} package \pkg{netgwas} provides a set of tools based
		on undirected graphical models for accomplishing three important
		and interrelated goals in genetics: (1) linkage map construction, 
		(2) reconstructing intra- and inter-chromosomal conditional
		interactions (linkage disequilibrium) networks, and (3) exploring 
		high-dimensional genotype-phenotype network and genotype-phenotype- 
		environment interactions network. The netgwas can deal with biparental 
		species with any  ploidy level.
		The package implemented the recent improvements both for construction 
		of linkage maps in diploid and polyploid species in Behrouzi and Wit(2017b),
		and in reconstructing networks for non-Gaussian data, ordinal data, and 
		mixed continuous and discrete data in Behrouzi and Wit (2017a). One 
		application is to uncover epistatic interactions network, where the network
		captures the conditionally dependent short- and long-range linkage disequilibrium
		structure of a genomes and reveals aberrant marker-marker associations.
		In addition, Behrouzi and Wit(2017c) implemented their proposed method to explore 
		genotype-phenotype networks where nodes are either phenotypes or genotypes, and each
		phenotype is connected by an edge to a genotype or a group of genotypes if 
		there is a direct association between them, given the rest of the variables. 
		Different phenotypes may also interconnect. The conditionally dependent relationships
		between markers on a genome and phenotypes is determined through Gaussian copula graphical model. 
		We remark that environmental variables can also be included along with genotype-phenotype
		input data to reconstruct networks between genotypes, phenotypes, and environment 
		variables. Beside, the package contains functions for simulation and visualization,
		as well as three multivariate datasets taken from literature.
}

\author{Pariya Behrouzi and Ernst C. Wit \cr
Maintainers: Pariya Behrouzi \email{pariya.behrouzi@gmail.com} \cr
}

\references{
1. Behrouzi, P., and Wit, E. C. (2019). Detecting epistatic selection with partially observed genotype data by using copula graphical models. Journal of the Royal Statistical Society: Series C (Applied Statistics), 68(1), 141-160. \cr
2. Behrouzi, P., and Wit, E. C. (2018). De novo construction of polyploid linkage maps using discrete graphical models. Bioinformatics. \cr
3. Behrouzi, P., Arends, D., and Wit, E. C. (2023). netgwas: An R Package for Network-Based Genome-Wide Association Studies. The R journal, 14(4), 18-37.
}

\examples{
\dontrun{
install.packages("netgwas")
library(netgwas)
}
}
\keyword{package}

\name{selectnet}
\alias{selectnet}

\title{ Model selection }

\description{
Estimate the optimal regularization parameter at EM convergence based on different information criteria .
}

\usage{
selectnet(netgwas.obj, opt.index= NULL, criteria= NULL, ebic.gamma=0.5,
		   ncores= NULL, verbose= TRUE)
}

\arguments{
	\item{netgwas.obj}{An object with S3 class "netgwas"}
	\item{opt.index}{The program internally determines an optimal graph using \code{opt.index= NULL}. Otherwise, to manually choose an optimal graph from the graph path.}
	\item{criteria}{Model selection criteria. "ebic" and "aic" are available. BIC model selection can be calculated by fixing \code{ebic.gamma = 0}. Applicable only if \code{opt.index= NULL}.}
	\item{ebic.gamma}{The tuning parameter for ebic. The\code{ebic.gamma = 0} results in bic model selection. The default value is 0.5. Applicable only \code{opt.index= NULL}.}
	\item{ncores}{ The number of cores to use for the calculations. Using \code{ncores = "all"} automatically detects number of available cores and runs the computations in parallel.}
	\item{verbose}{If \code{verbose = FALSE}, printing information is disabled. The default value is \code{TRUE}. Applicable only \code{opt.index= NULL}.}
}

\details{
This function computes extended Bayesian information criteria (ebic), Bayesian information criteria, Akaike information criterion (aic) at EM convergence based on observed or joint log-likelihood.
The observed log-likelihood can be obtained through

\deqn{\ell_Y(\widehat{\Theta}_\lambda) = Q(\widehat{\Theta}_\lambda | \widehat{\Theta}^{(m)}) - H (\widehat{\Theta}_\lambda | \widehat{\Theta}^{(m)}),}

Where \eqn{Q} can be calculated from \code{\link{netmap}}, \code{\link{netsnp}}, \code{\link{netphenogeno}} function and H function is
\deqn{H(\widehat{\Theta}_\lambda | \widehat{\Theta}^{(m)}_\lambda) = E_z[\ell_{Z | Y}(\widehat{\Theta}_\lambda) | Y; \widehat{\Theta}_\lambda] = E_z[\log f(z)| Y ;\widehat{\Theta}_\lambda ] - \log p(y).}

The "ebic" and "aic" model selection criteria can be obtained as follow
\deqn{ebic(\lambda) = -2 \ell(\widehat{\Theta}_\lambda) +  ( \log n + 4 \gamma \log p) df(\lambda)}

\deqn{aic(\lambda) = -2 \ell(\widehat{\Theta}_\lambda) +  2 df(\lambda)}
where \eqn{df} refers to the number of non-zeros offdiagonal elements of \eqn{\hat{\Theta}_\lambda}, and \eqn{\gamma \in [0, 1]}. Typical value for for \code{ebic.gamma} is 1/2, but it can also be tuned by experience. Fixing \code{ebic.gamma = 0 } results in bic model selection.
}

\value{
An obj with S3 class "selectnet" is returned:
 \item{opt.adj}{ The optimal graph selected from the graph path}
 \item{opt.theta}{The optimal precision matrix from the graph path}
 \item{opt.sigma}{ The optimal covariance matrix from the graph path}
 \item{ebic.scores}{ Extended BIC scores for regularization parameter selection at the EM convergence. Applicable if \code{opt.index = NULL}.}
 \item{opt.index}{ The index of optimal regularization parameter. }
 \item{opt.rho}{ The selected regularization parameter. }
 \item{par.cor}{ A partial correlation matrix. }
 \item{V.names}{ Variables name whose are not isolated. }

and anything else that is included in the input \code{netgwas.obj}.
 }

\references{
1. BBehrouzi, P., and Wit, E. C. (2019). Detecting epistatic selection with partially observed genotype data by using copula graphical models. Journal of the Royal Statistical Society: Series C (Applied Statistics), 68(1), 141-160. \cr
2. Behrouzi, P., Arends, D., and Wit, E. C. (2023). netgwas: An R Package for Network-Based Genome-Wide Association Studies. The R journal, 14(4), 18-37. \cr
3. Ibrahim, Joseph G., Hongtu Zhu, and Niansheng Tang. (2012). Model selection criteria for missing-data problems using the EM algorithm. Journal of the American Statistical Association.
4. D. Witten and J. Friedman. (2011). New insights and faster computations for the graphical lasso. \emph{Journal of Computational and Graphical Statistics}, to appear. \cr
5. J. Friedman, T. Hastie and R. Tibshirani. (2007). Sparse inverse covariance estimation with the lasso, \emph{Biostatistics}.\cr
6. Foygel, R. and M. Drton. (2010). Extended bayesian information criteria for Gaussian graphical models. In Advances in Neural Information Processing Systems, pp. 604-612.
}

\author{
  Pariya Behrouzi and Ernst C.Wit \cr
  Maintainer: Pariya Behrouzi \email{pariya.behrouzi@gmail.com} \cr
}

\seealso{\code{\link{netmap}}, \code{\link{netsnp}}, \code{\link{netphenogeno}}}

\examples{
	\dontshow{
		D <- simgeno(p=20, n=10, k= 3, adjacent = 1, alpha = 0.06 , beta = 0.3)
		out <- netsnp(data = D$data, n.rho= 2, ncores= 1)
		plot(out)
		sel <- selectnet(out, criteria = "ebic")
    }
	\donttest{
		#simulate data
		D <- simgeno(p=50, n=100, k= 3, adjacent = 3, alpha = 0.06 , beta = 0.06)
		plot(D)

		#explore intra- and inter-chromosomal interactions
		out  <-  netsnp(D$data, n.rho= 5, ncores= 1)
		plot(out)

		#different graph selection methods
		sel.ebic1 <- selectnet(out, criteria = "ebic")
		plot(sel.ebic1, vis = "CI")

		sel.aic <- selectnet(out, criteria = "aic")
		plot(sel.aic, vis = "CI")

		sel.bic <- selectnet(out, criteria = "ebic", ebic.gamma = 0)
		plot(sel.bic, vis = "CI")
	}
}

\name{simRIL}
\alias{simRIL}

\title{Generate genotype data of RIL}

\description{
Generating genotype data from a recombinant inbred line (RIL) population.
}

\usage{
simRIL( d = 25, n = 200, g = 5, cM = 100, selfing=2 )
}

\arguments{
	\item{d}{The number of markers per chromosome. The default value is 25.}
	\item{n}{The number of sample size (observations). The default value is 200.}
	\item{g}{The number of linkage groups (chromosomes). The default value is 5.}
	\item{cM}{The length of each chromosome based on centiMorgan.}
	\item{selfing}{ The number of selfing in RIL population.}
}

\value{
 \item{data}{The generated RIL genotype data as an \code{n} by \code{(d x g)} matrix.}
 \item{map}{The genetic map of the data.}
}

\author{ 
  Pariya Behrouzi \cr
  Maintainer: Pariya Behrouzi <pariya.behrouzi@gmail.com>
}

\seealso{
\code{\link{netmap}}, \code{\link{netsnp}}, and \code{\link{netgwas-package}}
}

\examples{
#genome-like graph structure
ril <- simRIL(g = 5, d = 25, cM = 100, n = 200, selfing = 2)
geno <- ril$data; image(geno, xlab= "individuals", ylab="markers")
map <- ril$map 
}

\name{netsnp}
\alias{netsnp}

\title{Reconstructs intra- and inter- chromosomal conditional interactions among genetic loci}

\description{
This is one of the main functions of the \pkg{netgwas} package. This function can be used to reconstruct the intra- and inter-chromosomal interactions among genetic loci in diploids and polyploids. The input data can be belong to any biparental genotype data which contains at least two genotype states. Two methods are available to reconstruct the network, namely (1) approximation method, and (2) gibbs sampling within the Gaussian copula graphical model. Both methods are able to deal with missing genotypes.
}

\usage{
netsnp(data, method = "gibbs", rho = NULL, n.rho = NULL, rho.ratio = NULL, 
		ncores = 1, em.iter = 5, em.tol = .001, verbose = TRUE)
}

\arguments{
	\item{data}{ An (\eqn{n \times p}) matrix or a \code{data.frame} corresponding to a genotype data matrix (\eqn{n} is the sample size and \eqn{p} is the number of variables). It also could be an object of class \code{"simgeno"}. Input data can contain missing values.}
	\item{method}{ Reconstructs intra- and inter- chromosomal conditional interactions (epistatic selection) network with three methods: "gibbs", "approx", and "npn". For a medium (~500) and a large number of variables we would recommend to choose "gibbs" and "approx", respectively. For a very large number of variables (> 2000) choose "npn". The default method is "gibbs".}
	\item{rho}{ A decreasing sequence of non-negative numbers that control the sparsity level. Leaving the input as \code{rho = NULL}, the program automatically computes a sequence of \code{rho} based on \code{n.rho} and \code{rho.ratio}. Users can also supply a decreasing sequence values to override this.}
	\item{n.rho}{ The number of regularization parameters. The default value is \code{10}.}
	\item{rho.ratio}{Determines the distance between the elements of \code{rho} sequence. A small value of \code{rho.ratio} results in a large distance between the elements of \code{rho} sequence. And a large value of \code{rho.ratio} results into a small distance between elements of \code{rho}. If keep it as NULL the program internally chooses a value.}
	\item{ncores}{ The number of cores to use for the calculations. Using \code{ncores = "all"} automatically detects number of available cores and runs the computations in parallel on (available cores - 1).}
	\item{em.iter}{ The number of EM iterations. The default value is 5.}
	\item{em.tol}{ A criteria to stop the EM iterations. The default value is .001. }
	\item{verbose}{Providing a detail message for tracing output. The default value is \code{TRUE}.}
}

\details{
Viability is a phenotype that can be considered. This function detects the conditional dependent short- and long-range linkage disequilibrium structure of genomes and thus reveals aberrant marker-marker associations that are due to epistatic selection.
This function can be used to estimate conditional independence relationships between partially observed data that not follow Gaussianity assumption (e.g. continuous non-Gaussian, discrete, or mixed dataset).
}

\value{
An object with S3 class \code{"netgwas"} is returned:  
 \item{Theta}{ A list of estimated p by p precision matrices that show the conditional independence relationships patterns among genetic loci.}
 \item{path}{ A list of estimated p by p adjacency matrices. This is the graph path corresponding to \code{Theta}.}
 \item{ES}{ A list of estimated p by p conditional expectation corresponding to \code{rho}.}
 \item{Z}{ A list of n by p transformed data based on Gaussian copula. }
 \item{rho}{ A \code{n.rho} dimensional vector containing the penalty terms.}
 \item{loglik }{ A \code{n.rho} dimensional vector containing the maximized log-likelihood values along the graph path. }
 \item{data}{ The \eqn{n} by \eqn{p} input data matrix. }
}

\note{ This function estimates a graph path . To select an optimal graph please refer to \code{\link{selectnet}}.}

\author{ 
  Pariya Behrouzi and Ernst C. Wit \cr
  Maintainers: Pariya Behrouzi \email{pariya.behrouzi@gmail.com}\cr
}

\references{
1. Behrouzi, P., and Wit, E. C. (2019). Detecting epistatic selection with partially observed genotype data by using copula graphical models. Journal of the Royal Statistical Society: Series C (Applied Statistics), 68(1), 141-160. \cr
2. Behrouzi, P., and Wit, E. C. (2017c). netgwas: An R Package for Network-Based Genome-Wide Association Studies. arXiv preprint, arXiv:1710.01236.
3. D. Witten and J. Friedman. New insights and faster computations for the graphical lasso. \emph{Journal of Computational and Graphical Statistics}, to appear, 2011. \cr
4. Guo, Jian, et al. "Graphical models for ordinal data." Journal of Computational and Graphical Statistics 24.1 (2015): 183-204. \cr
}

\seealso{\code{\link{selectnet}}}

\examples{
    \dontshow{
		D <- simgeno(p=20, n=10, k= 3, adjacent = 1, alpha = 0.06 , beta = 0.3)
		out <- netsnp(data = D$data, n.rho= 2, ncores= 1)
		plot(out)
		sel <- selectnet(out)
       }
	\donttest{
		data(CviCol)
		out <- netsnp(CviCol); out
		plot(out)
		
		#select optimal graph
		epi <- selectnet(out)
		plot(epi, vis="CI", xlab="markers", ylab="markers", 
		    n.mem = c(24,14,17,16,19), vertex.size=4)
		    
		#Visualize interactive plot of the selected network
		#Different colors for each chromosome
		cl <- c(rep("red", 24), rep("white",14), rep("tan1",17), 
		      rep("gray",16), rep("lightblue2",19))
		plot(epi, vis="interactive", vertex.color= cl)
		
		#Partial correlations between markers on genome
		image(as.matrix(epi$par.cor), xlab="markers", ylab="markers", sub="")
	}	
}

\name{plot.netgwas}
\alias{plot.netgwas}

\title{ plot for S3 class "netgwas" }

\description{
Plot the graph path which is the output of two functions \code{\link{netsnp}}, \code{\link{netphenogeno}}.
}

\usage{
\method{plot}{netgwas}( x, n.markers=NULL , ... )
}

\arguments{
	\item{x}{An object from "netgwas" class.}
	\item{n.markers}{A vector containing number of variables/markers in each group/chromosome. For example, the CviCol dataset that is provided in the package contains 5 chromosomes/ groups which the total number of markers is \eqn{p = 90}, where the first 24 markers belong into chromosome 1, the next 14 markers into chromosome 2, ..., and chromosome 5 contains 19 markers. Thus, n.mrkr = c(24,14,17,16,19). If \code{n.mrkr = NULL}, in the graph visualization all markers are represented same colour.}
	\item{\dots}{System reserved (No specific usage)}
}

\author{ 
  Pariya Behrouzi and Ernst C. Wit \cr
  Maintainer: Pariya Behrouzi \email{pariya.behrouzi@gmail.com}
}

\references{
Behrouzi, P., and Wit, E. C. (2017c). netgwas: An R Package for Network-Based Genome-Wide Association Studies. arXiv preprint, arXiv:1710.01236.
}


\seealso{\code{\link{netmap}}, \code{\link{netsnp}}, \code{\link{netphenogeno}}.}

\name{lower.upper}
\alias{lower.upper}

\title{
Calculates lower band and upper band
}

\description{
Calculates lower and upper bands for each data point, using a set of cut-points which is obtained from the Gaussian copula.
}

\usage{
lower.upper(y)
}

\arguments{
	\item{y}{ An (\eqn{n \times p}) matrix or a \code{data.frame} corresponding to the data matrix (\eqn{n} is the sample size and \eqn{p} is the number of variables). It also could be an object of class \code{"episim"}.
}
}

\value{
\item{lower}{
A \eqn{n} by \eqn{p} matrix representing the lower band for each data point.
}
\item{upper}{
A \eqn{n} by \eqn{p} matrix representing the upper band for each data point.
}
}

\author{ 
  Pariya Behrouzi and Ernst C. Wit \cr
  Maintainer: Pariya Behrouzi <pariya.behrouzi@gmail.com>
}

\references{
Behrouzi, P., and Wit, E. C. (2019). Detecting epistatic selection with partially observed genotype data by using copula graphical models. Journal of the Royal Statistical Society: Series C (Applied Statistics), 68(1), 141-160. \cr
}

\seealso{
  \code{\link{cutoffs}} and \code{\link{netgwas-package}}.
}

\examples{
D <- simgeno(p = 100, n = 50, k = 3)
lower.upper(D$data)
}

\name{R.gibbs}
\alias{R.gibbs}

\title{ The expectation of covariance matrix using Gibbs sampling}

\description{
This function implements the Gibbs sampling method within Gaussian copula graphical model to estimate the conditional expectation for the data that not follow Gaussianity assumption (e.g. ordinal, discrete, continuous non-Gaussian, or mixed dataset).
}

\usage{
R.gibbs(y, theta, gibbs.iter = 1000, mc.iter = 500, 
                   ncores = NULL, verbose = TRUE)
}

\arguments{
	\item{y}{
	 An (\eqn{n \times p}) matrix or a \code{data.frame} corresponding to the data matrix (\eqn{n} is the sample size and \eqn{p} is the number of variables). 
	 It also could be an object of class \code{"simgeno"}.}
	\item{theta}{ A \eqn{p \times p} precision matrix. Default is a diagonal matrix.}
	\item{gibbs.iter}{ The number of burn-in for the Gibbs sampling. The default value is 1000.}
	\item{mc.iter}{ The number of Monte Carlo samples to calculate the conditional expectation. The default value is 500. }
	\item{ncores}{If \code{ncores = NULL}, the algorithm internally detects number of available cores and run the calculations in parallel on (available cores - 1). Typical usage is to fix \code{ncores = 1} when \eqn{p} is small \eqn{( p < 500 )}, and \code{ncores = NULL} when \eqn{p} is very large.}
	\item{verbose}{ If \code{verbose = FALSE}, printing information is disabled. The default value is \code{TRUE}.}
}

\details{
This function calculates \eqn{\bar{R}} using Gibbs sampling method within the E-step of EM algorithm, where 
\deqn{ \bar{R} = n ^ {-1} \sum_{i=1}^{n} E( Z^{(i)} Z^{(i)t} | y^{(i)}, \hat{\Theta}^{(m)})}
which \eqn{n} is the number of sample size and \eqn{Z} is the latent variable which is obtained from Gaussian copula graphical model.
}

\value{
\item{ES}{Expectation of covariance matrix ( diagonal scaled to 1) of the Gaussian copula graphical model}
}

\references{
1. Behrouzi, P., Arends, D., and Wit, E. C. (2023). netgwas: An R Package for Network-Based Genome-Wide Association Studies. The R journal, 14(4), 18-37.\cr
2. Behrouzi, P., and Wit, E. C. (2019). Detecting epistatic selection with partially observed genotype data by using copula graphical models. Journal of the Royal Statistical Society: Series C (Applied Statistics), 68(1), 141-160. \cr
}

\author{ 
  Pariya Behrouzi, Danny Arends and Ernst C. Wit \cr
  Maintainers: Pariya Behrouzi \email{pariya.behrouzi@gmail.com}\cr
}

\examples{
D <- simgeno(p = 100, n = 50, k = 3)
R.gibbs(D$data, ncores=1)
}

\name{R.approx}
\alias{R.approx}

\title{ The expectation of covariance using approximation method }

\description{
This function implements the approximation method within the Gaussian copula graphical model to estimate the conditional expectation for the data that not follow Gaussianity assumption (e.g. ordinal, discrete, continuous non-Gaussian, or mixed dataset).
}

\usage{
R.approx(y, Z = NULL, Sigma=NULL, rho = NULL, ncores = NULL )
}

\arguments{
	\item{y}{An (\eqn{n \times p}) matrix or a \code{data.frame} corresponding to the data matrix (\eqn{n} is the sample size and \eqn{p} is the number of variables). 
	 It also could be an object of class \code{"simgeno"}.}
	\item{Z}{ A (\eqn{n \times p}) matrix which is a transformation of the data via the Gaussian copula. If \code{Z = NULL} internally calculates an initial value for \eqn{Z}.}
	\item{Sigma}{The covariance matrix of the latent variable given the data. If \code{Sigma = NULL} the Sigma matrix is calculated internally with a desired penalty term, \code{rho}.}
	\item{rho}{A (non-negative) regularization parameter to calculate \code{Sigma}. rho=0 means no regularization.}
	\item{ncores}{If \code{ncores = NULL}, the algorithm internally detects number of available cores and run the calculations in parallel on (available cores - 1). Typical usage is to fix \code{ncores = 1} when \eqn{p} is small \eqn{( p < 500 )}, and \code{ncores = NULL} when \eqn{p} is large.}
}

\value{
\item{ES}{Expectation of covariance matrix( diagonal scaled to 1) of the Gaussian copula graphical model.}
\item{Z}{ New transformation of the data based on given or default \code{Sigma}.}
}

\author{ 
  Pariya Behrouzi and Ernst C. Wit \cr
  Maintainer: Pariya Behrouzi \email{pariya.behrouzi@gmail.com}
}

\references{
1. Behrouzi, P., Arends, D., and Wit, E. C. (2023). netgwas: An R Package for Network-Based Genome-Wide Association Studies. The R journal, 14(4), 18-37.\cr
2. Behrouzi, P., and Wit, E. C. (2019). Detecting epistatic selection with partially observed genotype data by using copula graphical models. Journal of the Royal Statistical Society: Series C (Applied Statistics), 68(1), 141-160. \cr
}

\examples{
\dontrun{
D <- simgeno(p = 90, n = 50, k = 3)
R.approx(D$data)
} 
}

\name{netmap}
\alias{netmap}

\title{Constructing linkage map for diploids and polyploids}

\description{
This is one of the main functions of \pkg{netgwas} package. This function reconstructs linkage maps for biparental diploid and polyploid organisms using three methods.
}

\usage{
netmap(data, method = "npn", cross= NULL, rho = NULL, n.rho = NULL, 
      rho.ratio = NULL, min.m= NULL, use.comu= FALSE, ncores = "all",
		  em.iter = 5, verbose = TRUE)
}

\arguments{
	\item{data}{ An (\eqn{n \times p}) matrix or a \code{data.frame} corresponding to a genotype data matrix (\eqn{n} is the sample size and \eqn{p} is the number of variables). Input data can contain missing values.}
	\item{method}{Three available methods to construct linkage map: "gibbs", "approx", and "npn". Default is "npn"}
	\item{rho}{ A decreasing sequence of non-negative numbers that control the sparsity level. Leaving the input as \code{rho = NULL}, the program automatically computes a sequence of \code{rho} based on \code{n.rho} and \code{rho.ratio}. Users can also supply a decreasing sequence values to override this.}
	\item{n.rho}{ The number of regularization parameters. The default value is \code{6}.}
	\item{rho.ratio}{ Determines distance between the elements of \code{rho} sequence. A small value of \code{rho.ratio} results in a large distance between the elements of \code{rho} sequence. And a large value of \code{rho.ratio} results into a small distance between elements of \code{rho}. If keep it as NULL the program internally chooses a value.}
	\item{cross}{To be specified either "inbred" or "outbred".}
	\item{min.m}{ Expected minimum number of markers in a chromosome. Optional}
	\item{use.comu}{Use community detection algorithm to detect linkage groups. Default is FALSE.}
	\item{ncores}{ The number of cores to use for the calculations. Using \code{ncores = "all"} automatically detects number of available cores and runs the computations in parallel on (available cores - 1).}
	\item{em.iter}{The number of EM iterations. The default value is 5.}
	\item{verbose}{Providing a detail message for tracing output. The default value is \code{TRUE}.}
}

\details{
Constructing linkage maps for diploid and polyploid organisms. Diploid organisms contain two sets of chromosomes, one from each parent, whereas polyploids contain more than two sets of chromosomes. Inbreeding is mating between two parental lines where they have recent common biological ancestors. If they have no common ancestors up to roughly e.g. 4-6 generations, this is called outcrossing. In both cases the genomes of the derived progenies are random mosaics of the genome of the parents. However, in the case of inbreeding parental alleles are distinguishable in the genome of the progeny; in outcrossing this does not hold.
}

\value{
An object with S3 class \code{"netgwasmap"} is returned:  
 \item{map}{Constructed linkage map.}
 \item{opt.index}{ The index of selected graph using model selection.}
 \item{cross}{The pre-specified cross type. }
 \item{allres}{ A list containing results for different regularization parameter. Belongs to class "netgwas". To visualize a path of different 3D maps consider function \code{\link{plot.netgwas}}. Note that the input data is reordered based on the estimated linkage map and is saved as data in this argument.}
\item{man}{Stays FALSE.}
}

\author{ 
  Pariya Behrouzi and Ernst C. Wit \cr
  Maintainers: Pariya Behrouzi \email{pariya.behrouzi@gmail.com}\cr
}

\references{
1. Behrouzi, P., and Wit, E. C. (2018). De novo construction of polyploid linkage maps using discrete graphical models. Bioinformatics. \cr
2. Behrouzi, Pariya, and Ernst C. Wit. "netgwas: An R Package for Network-Based Genome-Wide Association Studies." arXiv preprint arXiv:1710.01236 (2017). \cr
3. Guo, Jian, Elizaveta Levina, George Michailidis, and Ji Zhu. "Graphical models for ordinal data." Journal of Computational and Graphical Statistics 24, no. 1 (2015): 183-204. \cr
4. Liu, Han, Fang Han, Ming Yuan, John Lafferty, and Larry Wasserman. "High-dimensional semiparametric Gaussian copula graphical models." The Annals of Statistics 40, no. 4 (2012): 2293-2326. \cr 
5. Witten, Daniela M., Jerome H. Friedman, and Noah Simon. "New insights and faster computations for the graphical lasso." Journal of Computational and Graphical Statistics 20, no. 4 (2011): 892-900.
}

\examples{
\dontrun{
data(CviCol)
#Randomly change the order of markers across the genome
cvicol <- CviCol[ ,sample(ncol(CviCol))]
 
#Constructing linkage map using gibbs method
out <- netmap(cvicol, cross= "inbred", ncores=1); out
#Estimated linkage map
map <- out$map; map
#Plot the associated network
plot(out)
#Visualizing the path networks
plot(out$allres)
#Build a linkage map for 5th networks
bm <- buildMap(out, opt.index=5); bm
####################

#Constructing linkage map using approx method
out2 <- netmap(cvicol, method="approx", cross= "inbred", ncores=1); out2
#Estimated linkage map
map2 <- out2$map; map2
#Plot the related network
plot(out2)
#Visualize other networks
plot(out2$allres)
#Build a linkage map for 5th network
bm2 <- buildMap(out2, opt.index=5); bm2

#Constructing linkage map using npn method
out3 <- netmap(cvicol, method="npn", cross= "inbred", ncores=1); out3
#Estimated linkage map
map3 <- out3$map; map3
#Plot the related network
plot(out3)
}
}

\name{thaliana}
\alias{thaliana}
\docType{data}
\title{Arabidopsis thaliana phenotype and genotype data }
\description{ 
  The genotype data of the Kend-L x Col Recombinant Inbred Line (RIL) population along with flowering time and leaf numbers phenotype information.
}

\usage{data(thaliana)}
\format{
  The format is a matrix containing 181 single-nucleotide polymorphism (SNP) markers and 8 phenotypes information for 197 individuals. 
}  

\details{
The accession Kend-L (Kendalville-Lehle; Lehle-WT-16-03) is crossed to the common lab strain Col (Co\-lum\-bi\-a).
The resulting lines were taken through six rounds of selfing without any intentional selection. The resulting 282 KendC (Kend-L \eqn{\times} Col) 
lines were genotyped at 181 markers.  The flowering time was measured for 197 lines of this population in both long days, which promote rapid 
flowering in many A. \emph{thaliana} strains, and in short days. Flowering time was measured using days to flowering (DTF) as well as the 
total number of leaves (TLN), partitioned into rosette and cauline leaves. In total eight phenotypes have been measured, namely days to 
flowering (DTF), cauline leaf number (CLN), rosette leaf number (RLN), and total leaf number (TLN) in long days (LD), and DTF, CLN, RLN, 
and TLN in short days (SD). Thus, the final dataset consist of 197 observations for 189 variables (8 phenotypes and 181 genotypes - SNP markers) \cr
This data set can be used to reconstruct network among SNP markers and the measured phenotypes.
}

\source{
Balasubramanian, Sureshkumar, et al. (2009). "QTL mapping in new Arabidopsis thaliana advanced intercross-recombinant inbred lines." PLoS One 4.2: e4318.
}

\examples{
\dontrun{
data(thaliana)

# Graph path
out <- netphenogeno(thaliana, ncores=1)
plot(out)

sel <- selectnet(out)
plot(sel, vis= "interactive")
}
}

\keyword{datasets}

\name{simgeno}
\alias{simgeno}

\title{Generate genotype data based on Gaussian copula}

\description{
Generating discrete ordinal data based on underlying "genome-like" graph structure. The procedure of simulating data relies on a continues variable, which can be simulated from either multivariate normal distribution, or multivariate t-distribution with \code{d} degrees of freedom.   
}

\usage{
simgeno( p = 90, n = 200, k = NULL, g = NULL, adjacent = NULL, alpha =
              NULL , beta = NULL, con.dist = "Mnorm", d = NULL, vis = TRUE)
}

\arguments{
	\item{p}{The number of variables. The default value is 90.}
	\item{n}{The number of sample size (observations). The default value is 200.}
	\item{k}{The number of states (categories). The default value is 3.}
	\item{g}{The number of groups (chromosomes) in the graph. The default value is about \eqn{p/20} if \eqn{p >= 40} and 2 if \eqn{p < 40}.}
	\item{adjacent}{ The number of adjacent variable(s) to be linked to a variable. For example, if \code{adjacent = 1} indicates a variable is linked via an edge with its adjacent variable on the left hand side, and its adjacent variable on the right hand side. The \code{adjacent = 2} defines a variable is linked via an edge with its 2 adjacent variables on its left hand side, and 2 adjacent variables on its right hand side. The default value is 1.}
	\item{alpha}{ A probability that a pair of non-adjacent variables in the same group is given an edge. The default value is 0.01.}
	\item{beta}{ A probability that variables in different groups are linked with an edge. The default value is 0.02.}
	\item{con.dist}{ The distribution of underlying continuous variable. If \code{con.dist = "Mnorm"}, a multivariate normal distribution with mean 0 is applied. If \code{con.dist = "Mt"}, the t-distribution with \code{a} degrees of freedom is applied. 
	The default distribution is \code{con.dist = "Mnorm"}.}
	\item{d}{The degrees of freedom of the continuous variable, only applicable when 
	\code{con.dist = "Mt"}. The default value is 3.}
	\item{vis}{ Visualize the graph pattern and the adjacency matrix of the true graph structure. The default value is TRUE.}
}

\details{
The graph pattern is generated as below: \cr\cr
genome-like: \code{p} variables are evenly partitions variables into \code{g} disjoint groups; the adjacent variables within each group are linked via an edge. With a probability \code{alpha} a pair of non-adjacent variables in the same group is given an edge. Variables in different groups are linked with an edge with a probability of \code{beta}.
}

\value{
An object with S3 class "simgeno" is returned: \cr \cr
 \item{data}{The generated data as an \code{n} by \code{p} matrix.}
 \item{Theta}{A \code{p} by \code{p} matrix corresponding to the inverse of covariance.}
 \item{adj}{ A \code{p} by \code{p} matrix corresponding to the adjacency matrix of the true graph structure.}
 \item{Sigma}{ A \code{p} by \code{p} covariance matrix for the generated data.}
 \item{n.groups}{ The number of groups.}
 \item{groups}{A vector that indicates each variable belongs to which group.}
 \item{sparsity}{The sparsity levels of the true graph.}
}

\author{ 
  Pariya Behrouzi and Ernst C. Wit \cr
  Maintainer: Pariya Behrouzi <pariya.behrouzi@gmail.com>
}

\references{
1. Behrouzi, P., Arends, D., and Wit, E. C. (2023). netgwas: An R Package for Network-Based Genome-Wide Association Studies. The R journal, 14(4), 18-37.\cr
2. Behrouzi, P., and Wit, E. C. (2019). Detecting epistatic selection with partially observed genotype data by using copula graphical models. Journal of the Royal Statistical Society: Series C (Applied Statistics), 68(1), 141-160. \cr
}

\seealso{
\code{\link{netsnp}}, and \code{\link{netgwas-package}}
}

\examples{
#genome-like graph structure
sim1 <- simgeno(alpha = 0.01, beta = 0.02)
plot(sim1)

#genome-like graph structure with more edges between variables in a same or different groups
sim2 <- simgeno(adjacent = 3, alpha = 0.02 , beta = 0.03)
plot(sim2)
\donttest{
		#simulate data
		D <- simgeno(p=50, n=100, g=5, k= 3, adjacent = 3, alpha = 0.06 , beta = 0.08)
		plot(D)

		#Reconstructing intra- and inter-chromosomal conditional interactions (LD) network
		out <- netsnp(data = D$data, n.rho= 4, ncores= 1)
		plot(out)
		#Select an optimal graph
		sel <- selectnet(out)
		plot(sel, vis= "CI" )
	}
}

\name{cutoffs}
\alias{cutoffs}

\title{
Cut-points 
}

\description{
Calculates cut-points of ordinal variables with respect to the Gaussian copula. 
}

\usage{
cutoffs(y)
}

\arguments{
  \item{y}{
An (\eqn{n \times p}) matrix or a \code{data.frame} corresponding to the data matrix (\eqn{n} is the sample size and \eqn{p} is the number of variables). It also could be an object of class \code{"simgeno"}.
}
}

\details{ The relationship between \eqn{j}th variable and \eqn{j}th latent variable is expressed through this set of cut-points.
}

\value{
\item{cutoffs}{ A \eqn{p} by \eqn{(k + 1)} matrix representing the cut-point values under the Gaussian copula, where \eqn{k} defines the number of categories in the dataset.}
}

\author{ 
  Pariya Behrouzi and Ernst C. Wit \cr
  Maintainer: Pariya Behrouzi <pariya.behrouzi@gmail.com>
}

\references{
1. Behrouzi, P., and Wit, E. C. (2019). Detecting epistatic selection with partially observed genotype data by using copula graphical models. Journal of the Royal Statistical Society: Series C (Applied Statistics), 68(1), 141-160. \cr
2. Behrouzi, P., and Wit, E. C. (2018). De novo construction of polyploid linkage maps using discrete graphical models. Bioinformatics. \cr
3. Behrouzi, P., and Wit, E. C. (2017c). netgwas: An R Package for Network-Based Genome-Wide Association Studies. arXiv preprint, arXiv:1710.01236.
}

\seealso{
  \code{\link{lower.upper}}, \code{\link{simgeno}} and \code{\link{netgwas-package}}.
}

\examples{
	D <- simgeno(p = 100, n = 50, k = 3)
	cutoffs(D$data)
}

\name{plot.simgeno}
\alias{plot.simgeno}

\title{ Plot function for \code{S3} class \code{"simgeno"} }

\description{ Visualizes the pattern of the true graph, the adjacency matrix, precison matrix and the covariance matrix of the simulated data.}

\usage{
\method{plot}{simgeno}(x, layout = layout.fruchterman.reingold, ...)
}

\arguments{
	\item{x}{An object of \code{S3} class \code{"simgeno"}, from function \code{\link{simgeno}}. }
	\item{layout}{The default is \code{"layout.fruchterman.reingold"}.}
	\item{\dots}{ System reserved (No specific usage)}
}

\author{ 
  Pariya Behrouzi and Ernst C. Wit \cr
  Maintainer: Pariya Behrouzi \email{pariya.behrouzi@gmail.com} \cr
}

\references{
Behrouzi, P., and Wit, E. C. (2017c). netgwas: An R Package for Network-Based Genome-Wide Association Studies. arXiv preprint, arXiv:1710.01236.
}

\seealso{\code{\link{simgeno}}}

\examples{
\dontrun{
# Generating discrete ordinal data with "genome-like" graph structure
data.sim <- simgeno(alpha = 0.01, beta = 0.02)
plot( data.sim )
}
}

\name{tetraPotato}
\alias{tetraPotato}
\docType{data}
\title{tetraploid potato genotype data }

\description{ 
  Tetraploid potato (\emph{Solanum tuberosum L.}) genotype data.
}

\usage{data(tetraPotato)}
\format{
  The format is a matrix containing 1972 single-nucleotide polymorphism (SNP) markers for 156 individuals.
}  

\details{
The full-sib mapping population MSL603 consists of 156 F1 plants resulting from a cross
between female parent "Jacqueline Lee" and male parent "MSG227-2". The obtained genotype
data contain 1972 SNP markers with five allele dosages. This genotype data can be used to construct linkage map for tetraploid potato (see below example).
}

\source{
Massa, Alicia N., Norma C. Manrique-Carpintero, Joseph J. Coombs, Daniel G. Zarka, Anne E. Boone, William W. Kirk, Christine A. Hackett, Glenn J. Bryan, and David S. Douches. "Genetic linkage mapping of economically important traits in cultivated tetraploid potato (Solanum tuberosum L.)." G3: Genes, Genomes, Genetics 5, no. 11 (2015): 2357-2364.
}

\examples{
\donttest{
data(tetraPotato)
#Shuffle the order of markers
potato <- tetraPotato[ , sample(ncol(tetraPotato))]
#Constructing linkage map for tetraploid potato
out <- netmap(potato, cross = "outbred"); out
potato.map <- out$map; potato.map
#plot(out)
}
}

\keyword{datasets}
